@@ -20,7 +20,8 @@ package MongoDB::BSON;
 # ABSTRACT: Tools for serializing and deserializing data in BSON form
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
+
 
 use XSLoader;
 XSLoader::load("MongoDB", $VERSION);
@@ -32,9 +33,13 @@ use MongoDB::Error;
 use Moose;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use Type::Params qw/compile/;
+use boolean;
 use namespace::clean -except => 'meta';
 
+# cached for efficiency during decoding
+our $_boolean_true = true;
+our $_boolean_false = false;
+
 #pod =attr dbref_callback
 #pod
 #pod A document with keys C<$ref> and C<$id> is a special MongoDB convention
@@ -241,24 +246,16 @@ has prefer_numeric => (
 #pod
 #pod =cut
 
-
-my @encode_overrides =
-  qw/error_callback invalid_chars max_length op_char prefer_numeric/;
-my $encode_one_args;
-
 sub encode_one {
-    $encode_one_args ||= compile( Object, Any, Optional [HashRef]);
-    my ( $self, $document, $options ) = $encode_one_args->(@_);
+    my ( $self, $document, $options ) = @_;
 
-    for my $k (@encode_overrides) {
-        $options->{$k} = $self->$k unless exists $options->{$k};
-    }
+    my $merged_opts = { %$self, ( $options ? %$options : () ) };
 
-    my $bson = eval { MongoDB::BSON::_encode_bson( $document, $options ) };
-    if ( $@ or ( $options->{max_length} && length($bson) > $options->{max_length} ) ) {
-        my $msg = $@ || "Document exceeds maximum size $options->{max_length}";
-        if ( $options->{error_callback} ) {
-            $options->{error_callback}->( $msg, $document, 'encode_one' );
+    my $bson = eval { MongoDB::BSON::_encode_bson( $document, $merged_opts ) };
+    if ( $@ or ( $merged_opts->{max_length} && length($bson) > $merged_opts->{max_length} ) ) {
+        my $msg = $@ || "Document exceeds maximum size $merged_opts->{max_length}";
+        if ( $merged_opts->{error_callback} ) {
+            $merged_opts->{error_callback}->( $msg, $document, 'encode_one' );
         }
         else {
             Carp::croak("During encode_one, $msg");
@@ -286,32 +283,25 @@ sub encode_one {
 #pod
 #pod =cut
 
-my @decode_overrides =
-  qw/dbref_callback dt_type error_callback max_length/;
-my $decode_one_args;
-
 sub decode_one {
-    $decode_one_args ||= compile( Object, Str, Optional [HashRef] );
-    my ( $self, $string, $options ) = $decode_one_args->(@_);
+    my ( $self, $string, $options ) = @_;
 
-    for my $k ( @decode_overrides ) {
-        $options->{$k} = $self->$k unless exists $options->{$k};
-    }
+    my $merged_opts = { %$self, ( $options ? %$options : () ) };
 
-    if ( $options->{max_length} && length($string) > $options->{max_length} ) {
-        my $msg = "Document exceeds maximum size $options->{max_length}";
-        if ( $options->{error_callback} ) {
-            $options->{error_callback}->( $msg, \$string, 'decode_one' );
+    if ( $merged_opts->{max_length} && length($string) > $merged_opts->{max_length} ) {
+        my $msg = "Document exceeds maximum size $merged_opts->{max_length}";
+        if ( $merged_opts->{error_callback} ) {
+            $merged_opts->{error_callback}->( $msg, \$string, 'decode_one' );
         }
         else {
             Carp::croak("During decode_one, $msg");
         }
     }
 
-    my $document = eval { MongoDB::BSON::_decode_bson( $string, $options ) };
+    my $document = eval { MongoDB::BSON::_decode_bson( $string, $merged_opts ) };
     if ( $@ ) {
-        if ( $options->{error_callback} ) {
-            $options->{error_callback}->( $@, \$string, 'decode_one' );
+        if ( $merged_opts->{error_callback} ) {
+            $merged_opts->{error_callback}->( $@, \$string, 'decode_one' );
         }
         else {
             Carp::croak("During decode_one, $@");
@@ -340,7 +330,6 @@ sub clone {
     return $class->new( %$self, @args );
 }
 
-
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -357,7 +346,7 @@ MongoDB::BSON - Tools for serializing and deserializing data in BSON form
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
