@@ -18,11 +18,11 @@ package MongoDB::BSON::Binary;
 # ABSTRACT: MongoDB binary type
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 use Types::Standard -types;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 use constant {
     SUBTYPE_GENERIC            => 0,
@@ -64,8 +64,6 @@ has subtype => (
     default => MongoDB::BSON::Binary->SUBTYPE_GENERIC
 );
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
@@ -80,7 +78,7 @@ MongoDB::BSON::Binary - MongoDB binary type
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
