@@ -19,7 +19,7 @@ package MongoDB::BulkWrite;
 # ABSTRACT: MongoDB bulk write interface
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::Error;
 use MongoDB::OID;
@@ -29,6 +29,7 @@ use MongoDB::BulkWriteView;
 use Syntax::Keyword::Junction qw/any/;
 
 use Moose;
+use MongoDB::_Types -types, 'to_WriteConcern';
 use Types::Standard -types;
 use namespace::clean -except => 'meta';
 
@@ -142,7 +143,7 @@ sub find {
 #pod     $bulk->insert_one( $doc );
 #pod
 #pod Queues a document for insertion when L</execute> is called.  The document may
-#pod be a hash reference, an array reference (with balance key/value pairs) or a
+#pod be a hash reference, an array reference (with balanced key/value pairs) or a
 #pod L<Tie::IxHash> object.  If the document does not have an C<_id> field, one will
 #pod be added to the original.
 #pod
@@ -151,39 +152,18 @@ sub find {
 #pod =cut
 
 sub insert_one {
+    MongoDB::UsageError->throw("insert_one requires a single document reference as an argument")
+      unless @_ == 2 && ref( $_[1] );
+
     my ( $self, $doc ) = @_;
 
-    unless ( @_ == 2 && ref $doc eq any(qw/HASH ARRAY Tie::IxHash/) ) {
-        MongoDB::UsageError->throw("argument to insert_one must be a single hashref, arrayref or Tie::IxHash");
-    }
-
-    # clone docs to add id and leave original unmodified
     if ( ref $doc eq 'ARRAY' ) {
-        MongoDB::UsageError->throw("array reference to insert_one must have key/value pairs")
+        MongoDB::UsageError->throw("array reference to find must have key/value pairs")
           if @$doc % 2;
-        $doc = Tie::IxHash->new(@$doc);
-    }
-    elsif ( ref $doc eq 'HASH' ) {
-        $doc = Tie::IxHash->new(%$doc);
+        $doc = {@$doc};
     }
 
-    if ( $doc->Length && $doc->Keys(0) eq '_id' ) {
-        $self->_enqueue_write( [ insert => $doc ] );
-    }
-    else {
-        # XXX this encapsulation violation is ugly, but a lot faster than
-        # doing it element by element.
-        my $copy = bless [
-            { %{ $doc->[0] } },
-            [ @{ $doc->[1] } ],
-            [ @{ $doc->[2] } ],
-            0
-        ], "Tie::IxHash";
-        my $id = $copy->DELETE('_id');
-        $id = MongoDB::OID->new unless defined $id;
-        $copy->Unshift( _id => $id );
-        $self->_enqueue_write( [ insert => $copy ] );
-    }
+    $self->_enqueue_write( [ insert => $doc ] );
 
     return;
 }
@@ -227,7 +207,10 @@ sub insert_one {
 #pod =cut
 
 sub execute {
-    my ( $self, $write_concern ) = @_;
+    my ( $self, $write_concern  ) = @_;
+    $write_concern = to_WriteConcern($write_concern)
+        if defined($write_concern) && ref($write_concern) ne 'MongoDB::WriteConcern';
+
     if ( $self->_executed ) {
         MongoDB::UsageError->throw("bulk op execute called more than once");
     }
@@ -241,7 +224,7 @@ sub execute {
 
     $write_concern ||= $self->collection->write_concern;
 
-    my $op = MongoDB::Op::_BulkWrite->new(
+    my $op = MongoDB::Op::_BulkWrite->_new(
         db_name       => $self->_database->name,
         coll_name     => $self->collection->name,
         queue         => $self->_queue,
@@ -278,7 +261,7 @@ MongoDB::BulkWrite - MongoDB bulk write interface
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
@@ -369,7 +352,7 @@ An exception will be thrown on error.
     $bulk->insert_one( $doc );
 
 Queues a document for insertion when L</execute> is called.  The document may
-be a hash reference, an array reference (with balance key/value pairs) or a
+be a hash reference, an array reference (with balanced key/value pairs) or a
 L<Tie::IxHash> object.  If the document does not have an C<_id> field, one will
 be added to the original.
 
