@@ -20,7 +20,7 @@ package MongoDB::Collection;
 # ABSTRACT: A MongoDB Collection
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::Error;
 use MongoDB::IndexView;
@@ -30,6 +30,7 @@ use MongoDB::WriteConcern;
 use MongoDB::_Query;
 use MongoDB::Op::_Aggregate;
 use MongoDB::Op::_BatchInsert;
+use MongoDB::Op::_BulkWrite;
 use MongoDB::Op::_CreateIndexes;
 use MongoDB::Op::_Delete;
 use MongoDB::Op::_Distinct;
@@ -38,7 +39,6 @@ use MongoDB::Op::_ListIndexes;
 use MongoDB::Op::_Update;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use Type::Params qw/compile/;
 use Tie::IxHash;
 use Carp 'carp';
 use boolean;
@@ -159,6 +159,7 @@ has _client => (
     isa     => InstanceOf['MongoDB::MongoClient'],
     lazy    => 1,
     reader  => 'client',
+    init_arg => undef,
     builder => '_build__client',
 );
 
@@ -182,6 +183,7 @@ has _full_name => (
     isa     => Str,
     lazy    => 1,
     reader  => 'full_name',
+    init_arg => undef,
     builder => '_build__full_name',
 );
 
@@ -209,6 +211,7 @@ has _indexes => (
     isa     => InstanceOf['MongoDB::IndexView'],
     lazy    => 1,
     reader  => 'indexes',
+    init_arg => undef,
     builder => '_build__indexes',
 );
 
@@ -217,6 +220,28 @@ sub _build__indexes {
     return MongoDB::IndexView->new( collection => $self );
 }
 
+# these are constant, so we cache them
+has _op_args => (
+    is       => 'ro',
+    isa      => HashRef,
+    lazy     => 1,
+    init_arg => undef,
+    builder  => '_build__op_args',
+);
+
+sub _build__op_args {
+    my ($self) = @_;
+    return {
+        client          => $self->client,
+        db_name         => $self->database->name,
+        bson_codec      => $self->bson_codec,
+        coll_name       => $self->name,
+        write_concern   => $self->write_concern,
+        read_preference => $self->read_preference,
+        full_name       => join( ".", $self->database->name, $self->name ),
+    };
+}
+
 #--------------------------------------------------------------------------#
 # public methods
 #--------------------------------------------------------------------------#
@@ -286,7 +311,7 @@ sub with_codec {
 #pod     $id = $res->inserted_id;
 #pod
 #pod Inserts a single L<document|/Document> into the database and returns a
-#pod L<MongoDB::InsertOneResult> object.
+#pod L<MongoDB::InsertOneResult> or L<MongoDB::UnacknowledgedResult> object.
 #pod
 #pod If no C<_id> field is present, one will be added when a document is
 #pod serialized for the database without modifying the original document.
@@ -294,20 +319,17 @@ sub with_codec {
 #pod
 #pod =cut
 
-my $insert_one_args;
+# args not unpacked for efficiency; args are self, document
 sub insert_one {
-    $insert_one_args ||= compile( Object, IxHash);
-    my ( $self, $document ) = $insert_one_args->(@_);
+    MongoDB::UsageError->throw("document argument must be a reference")
+      unless ref( $_[1] );
 
-    my $op = MongoDB::Op::_InsertOne->new(
-        db_name       => $self->database->name,
-        bson_codec    => $self->bson_codec,
-        coll_name     => $self->name,
-        document      => $document,
-        write_concern => $self->write_concern,
+    return $_[0]->client->send_write_op(
+        MongoDB::Op::_InsertOne->_new(
+            document => $_[1],
+            %{ $_[0]->_op_args },
+        )
     );
-
-    return $self->client->send_write_op($op);
 }
 
 #pod =method insert_many
@@ -316,8 +338,9 @@ sub insert_one {
 #pod     $res = $coll->insert_many( [ @documents ], { ordered => 0 } );
 #pod
 #pod Inserts each of the L<documents|/Documents> in an array reference into the
-#pod database and returns a L<MongoDB::InsertManyResult>.  This is syntactic sugar
-#pod for doing a L<MongoDB::BulkWrite> operation.
+#pod database and returns a L<MongoDB::InsertManyResult> or
+#pod L<MongoDB::UnacknowledgedResult>.  This is syntactic sugar for doing a
+#pod L<MongoDB::BulkWrite> operation.
 #pod
 #pod If no C<_id> field is present, one will be added when a document is
 #pod serialized for the database without modifying the original document.
@@ -336,21 +359,23 @@ sub insert_one {
 #pod
 #pod =cut
 
-my $insert_many_args;
+# args not unpacked for efficiency; args are self, document, options
 sub insert_many {
-    $insert_many_args ||= compile( Object, ArrayRef[IxHash], Optional[HashRef] );
-    my ($self, $documents, $opts) = $insert_many_args->(@_);
+    MongoDB::UsageError->throw("documents argument must be an array reference")
+      unless ref( $_[1] ) eq 'ARRAY';
 
     # ordered defaults to true
-    my $ordered = ( defined $opts && exists $opts->{ordered} ) ? $opts->{ordered} : 1;
-
-    my $wc = $self->write_concern;
-    my $bulk = $ordered ? $self->ordered_bulk : $self->unordered_bulk;
-    $bulk->insert($_) for @$documents;
-    my $res = $bulk->execute( $wc );
-    return MongoDB::InsertManyResult->new(
-        acknowledged => $wc->is_acknowledged,
-        inserted     => $res->inserted,
+    return MongoDB::InsertManyResult->_new(
+        acknowledged => $_[0]->write_concern->is_acknowledged,
+        inserted     => $_[0]->client->send_write_op(
+            MongoDB::Op::_BulkWrite->_new(
+                queue => [ map { [ insert => $_ ] } @{ $_[1] } ],
+                ordered => ( ( defined $_[2] && exists $_[2]->{ordered} ) ? $_[2]->{ordered} : 1 ),
+                %{ $_[0]->_op_args },
+            )
+          )->inserted,
+        write_errors         => [],
+        write_concern_errors => [],
     );
 }
 
@@ -360,26 +385,22 @@ sub insert_many {
 #pod     $res = $coll->delete_one( { _id => $id } );
 #pod
 #pod Deletes a single document that matches a L<filter expression|/Filter expression> and returns a
-#pod L<MongoDB::DeleteResult> object.
+#pod L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult> object.
 #pod
 #pod =cut
 
-my $delete_one_args;
+# args not unpacked for efficiency; args are self, filter
 sub delete_one {
-    $delete_one_args ||= compile( Object, IxHash );
-    my ($self, $filter) = $delete_one_args->(@_);
+    MongoDB::UsageError->throw("filter argument must be a reference")
+      unless ref( $_[1] );
 
-    my $op = MongoDB::Op::_Delete->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        filter        => $filter,
-        just_one      => 1,
-        write_concern => $self->write_concern,
+    return $_[0]->client->send_write_op(
+        MongoDB::Op::_Delete->_new(
+            filter   => $_[1],
+            just_one => 1,
+            %{ $_[0]->_op_args },
+        )
     );
-
-    return $self->client->send_write_op( $op );
-
 }
 
 #pod =method delete_many
@@ -387,27 +408,24 @@ sub delete_one {
 #pod     $res = $coll->delete_many( $filter );
 #pod     $res = $coll->delete_many( { name => "Larry" } );
 #pod
-#pod Deletes all documents that match a L<filter expression|/Filter expression> and returns a
-#pod L<MongoDB::DeleteResult> object.
+#pod Deletes all documents that match a L<filter expression|/Filter expression>
+#pod and returns a L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult>
+#pod object.
 #pod
 #pod =cut
 
-my $delete_many_args;
+# args not unpacked for efficiency; args are self, filter
 sub delete_many {
-    $delete_many_args ||= compile( Object, IxHash );
-    my ($self, $filter) = $delete_many_args->(@_);
+    MongoDB::UsageError->throw("filter argument must be a reference")
+      unless ref( $_[1] );
 
-    my $op = MongoDB::Op::_Delete->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        filter        => $filter,
-        just_one      => 0,
-        write_concern => $self->write_concern,
+    return $_[0]->client->send_write_op(
+        MongoDB::Op::_Delete->_new(
+            filter   => $_[1],
+            just_one => 0,
+            %{ $_[0]->_op_args },
+        )
     );
-
-    return $self->client->send_write_op( $op );
-
 }
 
 #pod =method replace_one
@@ -415,8 +433,9 @@ sub delete_many {
 #pod     $res = $coll->replace_one( $filter, $replacement );
 #pod     $res = $coll->replace_one( $filter, $replacement, { upsert => 1 } );
 #pod
-#pod Replaces one document that matches a L<filter expression|/Filter expression>
-#pod and returns a L<MongoDB::UpdateResult> object.
+#pod Replaces one document that matches a L<filter expression|/Filter
+#pod expression> and returns a L<MongoDB::UpdateResult> or
+#pod L<MongoDB::UnacknowledgedResult> object.
 #pod
 #pod The replacement document must not have any field-update operators in it (e.g.
 #pod C<$set>).
@@ -427,24 +446,21 @@ sub delete_many {
 #pod
 #pod =cut
 
-my $replace_one_args;
+# args not unpacked for efficiency; args are self, filter, update, options
 sub replace_one {
-    $replace_one_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
-    my ($self, $filter, $replacement, $options) = $replace_one_args->(@_);
+    MongoDB::UsageError->throw("filter and replace arguments must be references")
+      unless ref( $_[1] ) && ref( $_[2] );
 
-    my $op = MongoDB::Op::_Update->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        filter        => $filter,
-        update        => $replacement,
-        multi         => false,
-        upsert        => $options->{upsert} ? true : false,
-        is_replace    => 1,
-        write_concern => $self->write_concern,
+    return $_[0]->client->send_write_op(
+        MongoDB::Op::_Update->_new(
+            filter     => $_[1],
+            update     => $_[2],
+            multi      => false,
+            upsert     => $_[3] && $_[3]->{upsert} ? true : false,
+            is_replace => 1,
+            %{ $_[0]->_op_args },
+        )
     );
-
-    return $self->client->send_write_op( $op );
 }
 
 #pod =method update_one
@@ -452,8 +468,9 @@ sub replace_one {
 #pod     $res = $coll->update_one( $filter, $update );
 #pod     $res = $coll->update_one( $filter, $update, { upsert => 1 } );
 #pod
-#pod Updates one document that matches a L<filter expression|/Filter expression> and
-#pod returns a L<MongoDB::UpdateResult> object.
+#pod Updates one document that matches a L<filter expression|/Filter expression>
+#pod and returns a L<MongoDB::UpdateResult> or L<MongoDB::UnacknowledgedResult>
+#pod object.
 #pod
 #pod The update document must have only field-update operators in it (e.g.
 #pod C<$set>).
@@ -465,24 +482,21 @@ sub replace_one {
 #pod
 #pod =cut
 
-my $update_one_args;
+# args not unpacked for efficiency; args are self, filter, update, options
 sub update_one {
-    $update_one_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
-    my ($self, $filter, $update, $options) = $update_one_args->(@_);
+    MongoDB::UsageError->throw("filter and update arguments must be references")
+      unless ref( $_[1] ) && ref( $_[2] );
 
-    my $op = MongoDB::Op::_Update->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        filter        => $filter,
-        update        => $update,
-        multi         => false,
-        upsert        => $options->{upsert} ? true : false,
-        is_replace    => 0,
-        write_concern => $self->write_concern,
+    return $_[0]->client->send_write_op(
+        MongoDB::Op::_Update->_new(
+            filter     => $_[1],
+            update     => $_[2],
+            multi      => false,
+            upsert     => $_[3] && $_[3]->{upsert} ? true : false,
+            is_replace => 0,
+            %{ $_[0]->_op_args },
+        )
     );
-
-    return $self->client->send_write_op( $op );
 }
 
 #pod =method update_many
@@ -491,7 +505,8 @@ sub update_one {
 #pod     $res = $coll->update_many( $filter, $update, { upsert => 1 } );
 #pod
 #pod Updates one or more documents that match a L<filter expression|/Filter
-#pod expression> and returns a L<MongoDB::UpdateResult> object.
+#pod expression> and returns a L<MongoDB::UpdateResult> or
+#pod L<MongoDB::UnacknowledgedResult> object.
 #pod
 #pod The update document must have only field-update operators in it (e.g.
 #pod C<$set>).
@@ -503,67 +518,21 @@ sub update_one {
 #pod
 #pod =cut
 
-my $update_many_args;
+# args not unpacked for efficiency; args are self, filter, update, options
 sub update_many {
-    $update_many_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
-    my ($self, $filter, $update, $options) = $update_many_args->(@_);
+    MongoDB::UsageError->throw("filter and update arguments must be references")
+      unless ref( $_[1] ) && ref( $_[2] );
 
-    my $op = MongoDB::Op::_Update->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        filter        => $filter,
-        update        => $update,
-        multi         => true,
-        upsert        => $options->{upsert} ? true : false,
-        is_replace    => 0,
-        write_concern => $self->write_concern,
+    return $_[0]->client->send_write_op(
+        MongoDB::Op::_Update->_new(
+            filter     => $_[1],
+            update     => $_[2],
+            multi      => true,
+            upsert     => $_[3] && $_[3]->{upsert} ? true : false,
+            is_replace => 0,
+            %{ $_[0]->_op_args },
+        )
     );
-
-    return $self->client->send_write_op( $op );
-}
-
-#pod =method save_one
-#pod
-#pod     $collection->save_one(
-#pod         {"_id" => MongoDB::OID->new, "author" => "joe"}
-#pod     );
-#pod     $post = $collection->find_one( {author => "joe"} );
-#pod
-#pod     $post->{phone} = "555-5555";
-#pod
-#pod     $collection->save_one( $post );
-#pod
-#pod Saves one document into the database, either replacing an existing one or
-#pod upserting a new one.  The document B<must> have an C<_id> field or an
-#pod exception will be thrown.
-#pod
-#pod The method returns a L<MongoDB::UpdateResult> object.
-#pod
-#pod Unlike an C<insert>, this method is idempotent and will not throw a
-#pod duplicate key exception from the C<_id> field.  (Other unique index
-#pod constraints could still be violated, resulting in an error.)
-#pod
-#pod Note: this method is merely syntactic sugar for a replace_one with upsert:
-#pod
-#pod     $collection->save_one( $doc );  # same as following replace_one:
-#pod     $collection->replace_one( { _id => $doc->{_id} }, $doc, { upsert => 1 } );
-#pod
-#pod =cut
-
-my $save_one_args;
-
-sub save_one {
-    $save_one_args ||= compile( Object, IxHash );
-    my ( $self, $document ) = $save_one_args->(@_);
-
-    if ( $document->EXISTS("_id") ) {
-        return $self->replace_one( { "_id" => $document->FETCH( "_id" ) },
-            $document, { upsert => true } );
-    }
-    else {
-        MongoDB::UsageError->throw("argument to save_one must have an '_id' field");
-    }
 }
 
 #pod =method find
@@ -627,34 +596,40 @@ sub save_one {
 #pod
 #pod =cut
 
-my $find_args;
 sub find {
-    $find_args ||= compile( Object, Optional[IxHash], Optional[HashRef] );
-    my ( $self, $filter, $options ) = $find_args->(@_);
+    my ( $self, $filter, $options ) = @_;
     $options ||= {};
 
     # backwards compatible sort option for deprecated 'query' alias
     $options->{sort} = delete $options->{sort_by} if $options->{sort_by};
 
     # possibly fallback to default maxTimeMS
-    if ( ! exists $options->{maxTimeMS} && $self->max_time_ms ) {
+    if ( !exists $options->{maxTimeMS} && $self->max_time_ms ) {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
     # coerce to IxHash
-    __ixhash($options, 'sort');
+    __ixhash( $options, 'sort' );
 
-    my $query = MongoDB::_Query->new(
-        %$options,
-        db_name         => $self->database->name,
-        coll_name       => $self->name,
-        bson_codec      => $self->bson_codec,
-        client          => $self->client,
-        read_preference => $self->read_preference,
-        filter          => $filter || {},
+    return MongoDB::Cursor->new(
+        query => MongoDB::_Query->_new(
+            modifiers           => {},
+            allowPartialResults => 0,
+            batchSize           => 0,
+            comment             => '',
+            cursorType          => 'non_tailable',
+            limit               => 0,
+            maxTimeMS           => 0,
+            noCursorTimeout     => 0,
+            oplogReplay         => 0,
+            projection          => undef,
+            skip                => 0,
+            sort                => undef,
+            %$options,
+            filter => $filter || {},
+            %{ $self->_op_args },
+        )
     );
-
-    return MongoDB::Cursor->new( query => $query );
 }
 
 #pod =method find_one
@@ -692,37 +667,36 @@ sub find {
 #pod
 #pod =cut
 
-my $find_one_args;
 sub find_one {
-    $find_one_args ||= compile( Object,
-        Optional [IxHash],
-        Optional [MaybeHashRef],
-        Optional [MaybeHashRef],
-    );
-    my ( $self, $filter, $projection, $options ) = $find_one_args->(@_);
+    my ( $self, $filter, $projection, $options ) = @_;
     $options ||= {};
 
     # possibly fallback to default maxTimeMS
-    if ( ! exists $options->{maxTimeMS} && $self->max_time_ms ) {
+    if ( !exists $options->{maxTimeMS} && $self->max_time_ms ) {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
     # coerce to IxHash
-    __ixhash($options, 'sort');
+    __ixhash( $options, 'sort' );
 
-    my $query = MongoDB::_Query->new(
+    return MongoDB::_Query->_new(
+        modifiers           => {},
+        allowPartialResults => 0,
+        batchSize           => 0,
+        comment             => '',
+        cursorType          => 'non_tailable',
+        limit               => 0,
+        maxTimeMS           => 0,
+        noCursorTimeout     => 0,
+        oplogReplay         => 0,
+        skip                => 0,
+        sort                => undef,
         %$options,
-        db_name         => $self->database->name,
-        coll_name       => $self->name,
-        bson_codec      => $self->bson_codec,
-        client          => $self->client,
-        read_preference => $self->read_preference,
-        filter          => $filter || {},
-        projection      => $projection || {},
-        limit           => -1,
-    );
-
-    return $query->execute->next;
+        filter     => $filter     || {},
+        projection => $projection || {},
+        limit      => -1,
+        %{ $self->_op_args },
+    )->execute->next;
 }
 
 #pod =method find_one_and_delete
@@ -748,10 +722,11 @@ sub find_one {
 #pod
 #pod =cut
 
-my $foad_args;
 sub find_one_and_delete {
-    $foad_args ||= compile( Object, IxHash, Optional[HashRef] );
-    my ( $self, $filter, $options ) = $foad_args->(@_);
+    MongoDB::UsageError->throw("filter argument must be a reference")
+      unless ref( $_[1] );
+
+    my ( $self, $filter, $options ) = @_;
 
     # rename projection -> fields
     $options->{fields} = delete $options->{projection} if exists $options->{projection};
@@ -806,10 +781,11 @@ sub find_one_and_delete {
 #pod
 #pod =cut
 
-my $foar_args;
 sub find_one_and_replace {
-    $foar_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
-    my ( $self, $filter, $replacement, $options ) = $foar_args->(@_);
+    MongoDB::UsageError->throw("filter and replace arguments must be references")
+      unless ref( $_[1] ) && ref( $_[2] );
+
+    my ( $self, $filter, $replacement, $options ) = @_;
 
     return $self->_find_one_and_update_or_replace($filter, $replacement, $options);
 }
@@ -847,8 +823,10 @@ sub find_one_and_replace {
 
 my $foau_args;
 sub find_one_and_update {
-    $foau_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
-    my ( $self, $filter, $update, $options ) = $foau_args->(@_);
+    MongoDB::UsageError->throw("filter and update arguments must be references")
+      unless ref( $_[1] ) && ref( $_[2] );
+
+    my ( $self, $filter, $update, $options ) = @_;
 
     return $self->_find_one_and_update_or_replace($filter, $update, $options);
 }
@@ -888,12 +866,21 @@ sub find_one_and_update {
 #pod See L<Aggregation|http://docs.mongodb.org/manual/aggregation/> in the MongoDB manual
 #pod for more information on how to construct aggregation queries.
 #pod
+#pod B<Note> The use of aggregation cursors is automatic based on your server
+#pod version.  However, if migrating a sharded cluster from MongoDB 2.4 to 2.6
+#pod or later, you must upgrade your mongod servers first before your mongos
+#pod routers or aggregation queries will fail.  As a workaround, you may
+#pod pass C<< cursor => undef >> as an option.
+#pod
 #pod =cut
 
 my $aggregate_args;
 sub aggregate {
-    $aggregate_args ||= compile( Object, ArrayOfHashRef, Optional [HashRef] );
-    my ( $self, $pipeline, $options ) = $aggregate_args->(@_);
+    MongoDB::UsageError->throw("pipeline argument must be an array reference")
+      unless ref( $_[1] ) eq 'ARRAY';
+
+    my ( $self, $pipeline, $options ) = @_;
+    $options ||= {};
 
     # boolify some options
     for my $k (qw/allowDiskUse explain/) {
@@ -909,7 +896,7 @@ sub aggregate {
     my ($last_op) = keys %{ $pipeline->[-1] };
     my $read_pref = $last_op eq '$out' ? undef : $self->read_preference;
 
-    my $op = MongoDB::Op::_Aggregate->new(
+    my $op = MongoDB::Op::_Aggregate->_new(
         db_name    => $self->database->name,
         coll_name  => $self->name,
         client     => $self->client,
@@ -948,11 +935,8 @@ sub aggregate {
 #pod
 #pod =cut
 
-my $count_args;
-
 sub count {
-    $count_args ||= compile( Object, Optional [IxHash], Optional [HashRef] );
-    my ( $self, $filter, $options ) = $count_args->(@_);
+    my ( $self, $filter, $options ) = @_;
     $filter  ||= {};
     $options ||= {};
 
@@ -973,9 +957,9 @@ sub count {
 
 #pod =method distinct
 #pod
-#pod     $result = $coll->count( $fieldname );
-#pod     $result = $coll->count( $fieldname, $filter );
-#pod     $result = $coll->count( $fieldname, $filter, $options );
+#pod     $result = $coll->distinct( $fieldname );
+#pod     $result = $coll->distinct( $fieldname, $filter );
+#pod     $result = $coll->distinct( $fieldname, $filter, $options );
 #pod
 #pod Returns a L<MongoDB::QueryResult> object that will provide distinct values for
 #pod a specified field name.
@@ -998,8 +982,10 @@ sub count {
 my $distinct_args;
 
 sub distinct {
-    $distinct_args ||= compile( Object, Str, Optional [IxHash], Optional [HashRef] );
-    my ( $self, $fieldname, $filter, $options ) = $distinct_args->(@_);
+    MongoDB::UsageError->throw("fieldname argument is required")
+      unless defined( $_[1] );
+
+    my ( $self, $fieldname, $filter, $options ) = @_;
     $filter ||= {};
     $options ||= {};
 
@@ -1008,7 +994,7 @@ sub distinct {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
-    my $op = MongoDB::Op::_Distinct->new(
+    my $op = MongoDB::Op::_Distinct->_new(
         db_name         => $self->database->name,
         coll_name       => $self->name,
         client          => $self->client,
@@ -1044,7 +1030,7 @@ sub parallel_scan {
     unless (defined $num_cursors && $num_cursors == int($num_cursors)
         && $num_cursors > 0 && $num_cursors <= 10000
     ) {
-        Carp::croak( "first argument to parallel_scan must be a positive integer between 1 and 10000" )
+        MongoDB::UsageError->throw( "first argument to parallel_scan must be a positive integer between 1 and 10000" )
     }
     $opts = ref $opts eq 'HASH' ? $opts : { };
 
@@ -1052,9 +1038,10 @@ sub parallel_scan {
 
     my @command = ( parallelCollectionScan => $self->name, numCursors => $num_cursors );
 
-    my $op = MongoDB::Op::_Command->new(
+    my $op = MongoDB::Op::_Command->_new(
         db_name         => $db->name,
         query           => \@command,
+        query_flags     => {},
         bson_codec      => $self->bson_codec,
         read_preference => $self->read_preference,
     );
@@ -1062,16 +1049,25 @@ sub parallel_scan {
     my $result = $self->client->send_read_op( $op );
     my $response = $result->output;
 
-    Carp::croak("No cursors returned")
+    MongoDB::UsageError->throw("No cursors returned")
         unless $response->{cursors} && ref $response->{cursors} eq 'ARRAY';
 
     my @cursors;
     for my $c ( map { $_->{cursor} } @{$response->{cursors}} ) {
-        my $qr = MongoDB::QueryResult->new(
+        my $batch = $c->{firstBatch};
+        my $qr = MongoDB::QueryResult->_new(
             _client    => $self->client,
             address    => $result->address,
+            ns         => $c->{ns},
             bson_codec => $self->bson_codec,
-            cursor     => $c,
+            batch_size   => scalar @$batch,
+            cursor_at    => 0,
+            limit        => 0,
+            cursor_id    => MongoDB::QueryResult::_pack_cursor_id($c->{id}),
+            cursor_start => 0,
+            cursor_flags => {},
+            cursor_num   => scalar @$batch,
+            _docs        => $batch,
         );
         push @cursors, $qr;
     }
@@ -1321,6 +1317,7 @@ sub _dynamic_write_concern {
 
 sub _find_one_and_update_or_replace {
     my ($self, $filter, $modifier, $options) = @_;
+    $options ||= {};
 
     # rename projection -> fields
     $options->{fields} = delete $options->{projection} if exists $options->{projection};
@@ -1359,10 +1356,11 @@ sub _run_command {
         $read_pref = MongoDB::ReadPreference->new($read_pref);
     }
 
-    my $op = MongoDB::Op::_Command->new(
-        db_name         => $self->database->name,
-        query           => $command,
-        bson_codec      => $self->bson_codec,
+    my $op = MongoDB::Op::_Command->_new(
+        db_name     => $self->database->name,
+        query       => $command,
+        query_flags => {},
+        bson_codec  => $self->bson_codec,
         ( $read_pref ? ( read_preference => $read_pref ) : () ),
     );
 
@@ -1415,14 +1413,14 @@ sub __ixhash {
 
 my $legacy_insert_args;
 sub insert {
-    $legacy_insert_args ||= compile( Object, IxHash, Optional[HashRef] );
-    my ( $self, $document, $opts ) = $legacy_insert_args->(@_);
+    MongoDB::UsageError->throw("document argument must be a reference")
+      unless ref( $_[1] );
 
-    my $op = MongoDB::Op::_InsertOne->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        document      => $document,
+    my ( $self, $document, $opts ) = @_;
+
+    my $op = MongoDB::Op::_InsertOne->_new(
+        document => $document,
+        %{ $self->_op_args },
         write_concern => $self->_dynamic_write_concern($opts),
     );
 
@@ -1431,17 +1429,20 @@ sub insert {
     return $result->inserted_id;
 }
 
-my $legacy_batch_args;
 sub batch_insert {
-    my ( $self, $documents, $opts ) = @_;
-    $legacy_batch_args ||= compile( Object, ArrayRef[IxHash], Optional[HashRef] );
+    MongoDB::UsageError->throw("documents argument must be an array reference")
+      unless ref( $_[1] ) eq 'ARRAY';
 
-    my $op = MongoDB::Op::_BatchInsert->new(
+    my ( $self, $documents, $opts ) = @_;
+
+    my $op = MongoDB::Op::_BatchInsert->_new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
         bson_codec    => $self->bson_codec,
         documents     => $documents,
         write_concern => $self->_dynamic_write_concern($opts),
+        check_keys    => 0,
+        ordered       => 1, 
     );
 
     my $result = $self->client->send_write_op($op);
@@ -1455,18 +1456,14 @@ sub batch_insert {
     return @ids;
 }
 
-my $legacy_remove_args;
 sub remove {
-    $legacy_remove_args ||= compile( Object, Optional[IxHash], Optional[HashRef] );
-    my ($self, $query, $opts) = $legacy_remove_args->(@_);
+    my ($self, $query, $opts) = @_;
     $opts ||= {};
 
-    my $op = MongoDB::Op::_Delete->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
+    my $op = MongoDB::Op::_Delete->_new(
         filter        => $query || {},
         just_one      => !! $opts->{just_one},
+        %{ $self->_op_args },
         write_concern => $self->_dynamic_write_concern($opts),
     );
 
@@ -1481,8 +1478,7 @@ sub remove {
 
 my $legacy_update_args;
 sub update {
-    $legacy_update_args ||= compile( Object, Optional[IxHash], Optional[IxHash], Optional[HashRef] );
-    my ( $self, $query, $object, $opts ) = $legacy_update_args->(@_);
+    my ( $self, $query, $object, $opts ) = @_;
     $opts ||= {};
 
     if ( exists $opts->{multiple} ) {
@@ -1494,54 +1490,73 @@ sub update {
     }
 
     # figure out if first key based on op_char or '$'
-    my $is_replace;
-    if ( $object && $object->Length ) {
-        my $fk = substr($object->FIRSTKEY, 0, 1);
-        my $op_char = eval { $self->bson_codec->op_char } || '';
-        $is_replace = $fk ne '$' && $fk ne $op_char;
-    }
+    my $type = ref($object);
+    my $fk = (
+          $type eq 'HASH' ? each(%$object)
+        : $type eq 'ARRAY' ? $object->[0]
+        : $type eq 'Tie::IxHash' ? $object->FIRSTKEY
+        : each (%$object)
+    );
+    $fk = defined($fk) ? substr($fk,0,1) : '';
 
-    my $op = MongoDB::Op::_Update->new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
-        filter        => $query || {},
-        update        => $object || {},
-        multi         => $opts->{multi},
-        upsert        => $opts->{upsert},
-        is_replace    => $is_replace,
+    my $op_char = eval { $self->bson_codec->op_char } || '';
+    my $is_replace = $fk ne '$' && $fk ne $op_char;
+
+    my $op = MongoDB::Op::_Update->_new(
+        filter => $query  || {},
+        update => $object || {},
+        multi  => $opts->{multi},
+        upsert => $opts->{upsert},
+        is_replace => $is_replace,
+        %{ $_[0]->_op_args },
         write_concern => $self->_dynamic_write_concern($opts),
     );
 
     my $result = $self->client->send_write_op( $op );
 
-    # emulate key fields of legacy GLE result
-    return {
-        ok => 1,
-        n => $result->matched_count,
-        ( $result->upserted_id ? ( upserted => $result->upserted_id ) : () ),
-    };
+    if ( $result->acknowledged ) {
+        # emulate key fields of legacy GLE result
+        return {
+            ok => 1,
+            n => $result->matched_count,
+            ( $result->upserted_id ? ( upserted => $result->upserted_id ) : () ),
+        };
+    }
+    else {
+        return { ok => 1 };
+    }
 }
 
-my $legacy_save_args;
 sub save {
-    $legacy_save_args ||= compile( Object, IxHash, Optional[HashRef] );
-    my ($self, $doc, $options) = $legacy_save_args->(@_);
+    MongoDB::UsageError->throw("document argument must be a reference")
+      unless ref( $_[1] );
 
-    if ( $doc->EXISTS("_id") ) {
+    my ($self, $doc, $options) = @_;
+
+    my $type = ref($doc);
+    my $id = (
+          $type eq 'HASH' ? $doc->{_id}
+        : $type eq 'ARRAY' ? do {
+            my $i;
+            for ( $i = 0; $i < @$doc; $i++ ) { last if $doc->[$i] eq '_id' }
+            $i < $#$doc ? $doc->[ $i + 1 ] : undef;
+          }
+        : $type eq 'Tie::IxHash' ? $doc->FETCH('_id')
+        : $doc->{_id} # hashlike?
+    );
+
+    if ( defined($id) ) {
         $options ||= {};
         $options->{'upsert'} = boolean::true;
-        return $self->update( { "_id" => $doc->FETCH( ("_id") ) }, $doc, $options );
+        return $self->update( { _id => $id }, $doc, $options );
     }
     else {
         return $self->insert( $doc, ( $options ? $options : () ) );
     }
 }
 
-my $legacy_fam_args;
 sub find_and_modify {
-    $legacy_fam_args ||= compile( Object, HashRef );
-    my ( $self, $opts ) = $legacy_fam_args->(@_);
+    my ( $self, $opts ) = @_;
 
     my $conn = $self->client;
     my $db   = $self->database;
@@ -1579,7 +1594,7 @@ sub ensure_index {
       ? $self->write_concern
       : MongoDB::WriteConcern->new;
 
-    my $op = MongoDB::Op::_CreateIndexes->new(
+    my $op = MongoDB::Op::_CreateIndexes->_new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
         bson_codec    => $self->bson_codec,
@@ -1653,7 +1668,7 @@ sub __to_index_string {
 sub get_indexes {
     my ($self) = @_;
 
-    my $op = MongoDB::Op::_ListIndexes->new(
+    my $op = MongoDB::Op::_ListIndexes->_new(
         db_name    => $self->database->name,
         coll_name  => $self->name,
         client     => $self->client,
@@ -1698,7 +1713,7 @@ MongoDB::Collection - A MongoDB Collection
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
@@ -1883,7 +1898,7 @@ to:
     $id = $res->inserted_id;
 
 Inserts a single L<document|/Document> into the database and returns a
-L<MongoDB::InsertOneResult> object.
+L<MongoDB::InsertOneResult> or L<MongoDB::UnacknowledgedResult> object.
 
 If no C<_id> field is present, one will be added when a document is
 serialized for the database without modifying the original document.
@@ -1895,8 +1910,9 @@ The generated C<_id> may be retrieved from the result object.
     $res = $coll->insert_many( [ @documents ], { ordered => 0 } );
 
 Inserts each of the L<documents|/Documents> in an array reference into the
-database and returns a L<MongoDB::InsertManyResult>.  This is syntactic sugar
-for doing a L<MongoDB::BulkWrite> operation.
+database and returns a L<MongoDB::InsertManyResult> or
+L<MongoDB::UnacknowledgedResult>.  This is syntactic sugar for doing a
+L<MongoDB::BulkWrite> operation.
 
 If no C<_id> field is present, one will be added when a document is
 serialized for the database without modifying the original document.
@@ -1919,23 +1935,25 @@ calls.
     $res = $coll->delete_one( { _id => $id } );
 
 Deletes a single document that matches a L<filter expression|/Filter expression> and returns a
-L<MongoDB::DeleteResult> object.
+L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult> object.
 
 =head2 delete_many
 
     $res = $coll->delete_many( $filter );
     $res = $coll->delete_many( { name => "Larry" } );
 
-Deletes all documents that match a L<filter expression|/Filter expression> and returns a
-L<MongoDB::DeleteResult> object.
+Deletes all documents that match a L<filter expression|/Filter expression>
+and returns a L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult>
+object.
 
 =head2 replace_one
 
     $res = $coll->replace_one( $filter, $replacement );
     $res = $coll->replace_one( $filter, $replacement, { upsert => 1 } );
 
-Replaces one document that matches a L<filter expression|/Filter expression>
-and returns a L<MongoDB::UpdateResult> object.
+Replaces one document that matches a L<filter expression|/Filter
+expression> and returns a L<MongoDB::UpdateResult> or
+L<MongoDB::UnacknowledgedResult> object.
 
 The replacement document must not have any field-update operators in it (e.g.
 C<$set>).
@@ -1949,8 +1967,9 @@ document will be upserted if no matching document exists.
     $res = $coll->update_one( $filter, $update );
     $res = $coll->update_one( $filter, $update, { upsert => 1 } );
 
-Updates one document that matches a L<filter expression|/Filter expression> and
-returns a L<MongoDB::UpdateResult> object.
+Updates one document that matches a L<filter expression|/Filter expression>
+and returns a L<MongoDB::UpdateResult> or L<MongoDB::UnacknowledgedResult>
+object.
 
 The update document must have only field-update operators in it (e.g.
 C<$set>).
@@ -1966,7 +1985,8 @@ operations to it prior to insertion.
     $res = $coll->update_many( $filter, $update, { upsert => 1 } );
 
 Updates one or more documents that match a L<filter expression|/Filter
-expression> and returns a L<MongoDB::UpdateResult> object.
+expression> and returns a L<MongoDB::UpdateResult> or
+L<MongoDB::UnacknowledgedResult> object.
 
 The update document must have only field-update operators in it (e.g.
 C<$set>).
@@ -1976,32 +1996,6 @@ C<upsert>, which defaults to false.  If provided and true, a new document will
 be inserted by taking the filter document and applying the update document
 operations to it prior to insertion.
 
-=head2 save_one
-
-    $collection->save_one(
-        {"_id" => MongoDB::OID->new, "author" => "joe"}
-    );
-    $post = $collection->find_one( {author => "joe"} );
-
-    $post->{phone} = "555-5555";
-
-    $collection->save_one( $post );
-
-Saves one document into the database, either replacing an existing one or
-upserting a new one.  The document B<must> have an C<_id> field or an
-exception will be thrown.
-
-The method returns a L<MongoDB::UpdateResult> object.
-
-Unlike an C<insert>, this method is idempotent and will not throw a
-duplicate key exception from the C<_id> field.  (Other unique index
-constraints could still be violated, resulting in an error.)
-
-Note: this method is merely syntactic sugar for a replace_one with upsert:
-
-    $collection->save_one( $doc );  # same as following replace_one:
-    $collection->replace_one( { _id => $doc->{_id} }, $doc, { upsert => 1 } );
-
 =head2 find
 
     $cursor = $coll->find( $filter );
@@ -2266,6 +2260,12 @@ and read preference is ignored.
 See L<Aggregation|http://docs.mongodb.org/manual/aggregation/> in the MongoDB manual
 for more information on how to construct aggregation queries.
 
+B<Note> The use of aggregation cursors is automatic based on your server
+version.  However, if migrating a sharded cluster from MongoDB 2.4 to 2.6
+or later, you must upgrade your mongod servers first before your mongos
+routers or aggregation queries will fail.  As a workaround, you may
+pass C<< cursor => undef >> as an option.
+
 =head2 count
 
     $count = $coll->count( $filter );
@@ -2303,9 +2303,9 @@ for details and a work-around using L</aggregate>.
 
 =head2 distinct
 
-    $result = $coll->count( $fieldname );
-    $result = $coll->count( $fieldname, $filter );
-    $result = $coll->count( $fieldname, $filter, $options );
+    $result = $coll->distinct( $fieldname );
+    $result = $coll->distinct( $fieldname, $filter );
+    $result = $coll->distinct( $fieldname, $filter, $options );
 
 Returns a L<MongoDB::QueryResult> object that will provide distinct values for
 a specified field name.
