@@ -19,15 +19,19 @@ package MongoDB::CommandResult;
 # ABSTRACT: MongoDB generic command result document
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 use MongoDB::Error;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
-with 'MongoDB::Role::_LastError';
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
+  MongoDB::Role::_LastError
+);
 
 #pod =attr output
 #pod
@@ -37,8 +41,8 @@ with 'MongoDB::Role::_LastError';
 
 has output => (
     is       => 'ro',
-    isa      => HashRef,
     required => 1,
+    isa => HashRef,
 );
 
 #pod =attr address
@@ -49,8 +53,8 @@ has output => (
 
 has address => (
     is       => 'ro',
-    isa      => HostAddress,
     required => 1,
+    isa => HostAddress,
 );
 
 #pod =method last_code
@@ -116,8 +120,6 @@ sub assert {
 # deprecated
 sub result { shift->output }
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
@@ -132,7 +134,7 @@ MongoDB::CommandResult - MongoDB generic command result document
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 DESCRIPTION
 
