@@ -20,15 +20,16 @@ package MongoDB::Cursor;
 # ABSTRACT: A lazy cursor for Mongo query results
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use Moose;
 use MongoDB;
 use MongoDB::BSON;
 use MongoDB::Error;
 use MongoDB::QueryResult;
+use MongoDB::ReadPreference;
 use MongoDB::_Protocol;
-use MongoDB::_Types -types;
+use MongoDB::_Types -types, 'to_IxHash';
 use Types::Standard -types;
 use boolean;
 use Tie::IxHash;
@@ -160,7 +161,7 @@ sub sort {
     MongoDB::UsageError->throw("cannot set sort after querying")
       if $self->started_iterating;
 
-    $self->query->sort($order);
+    $self->query->sort( to_IxHash($order) );
     return $self;
 }
 
@@ -571,20 +572,23 @@ sub count {
     return $result ? $result->{n} : 0;
 }
 
+my $PRIMARY = MongoDB::ReadPreference->new;
+my $SEC_PREFERRED = MongoDB::ReadPreference->new( mode => 'secondaryPreferred' );
+
 sub slave_okay {
     my ($self, $value) = @_;
     MongoDB::UsageError->throw("cannot set slave_ok after querying")
       if $self->started_iterating;
 
-    if ( $value ) {
+    if ($value) {
         # if not 'primary', then slave_ok is already true, so leave alone
         if ( $self->query->read_preference->mode eq 'primary' ) {
             # secondaryPreferred is how mongos interpretes slave_ok
-            $self->query->read_preference('secondaryPreferred');
+            $self->query->read_preference( $SEC_PREFERRED );
         }
     }
     else {
-        $self->query->read_preference('primary');
+        $self->query->read_preference( $PRIMARY );
     }
 
     # XXX returning self is an API change but more consistent with other cursor methods
@@ -611,7 +615,7 @@ MongoDB::Cursor - A lazy cursor for Mongo query results
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
