@@ -20,15 +20,15 @@ package MongoDB::Database;
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
 use MongoDB::GridFS;
 use MongoDB::Op::_ListCollections;
+use MongoDB::ReadPreference;
 use MongoDB::_Query;
 use MongoDB::_Types -types;
-use Type::Params qw/compile/;
 use Types::Standard -types;
 use Carp 'carp';
 use boolean;
@@ -157,8 +157,7 @@ has bson_codec => (
 my $list_collections_args;
 
 sub list_collections {
-    $list_collections_args ||= compile( Object, Optional [IxHash], Optional [HashRef] );
-    my ( $self, $filter, $options ) = $list_collections_args->(@_);
+    my ( $self, $filter, $options ) = @_;
     $filter  ||= {};
     $options ||= {};
 
@@ -167,7 +166,7 @@ sub list_collections {
         $options->{maxTimeMS} = $self->max_time_ms;
     }
 
-    my $op = MongoDB::Op::_ListCollections->new(
+    my $op = MongoDB::Op::_ListCollections->_new(
         db_name    => $self->name,
         client     => $self->_client,
         bson_codec => $self->bson_codec,
@@ -193,11 +192,12 @@ sub list_collections {
 sub collection_names {
     my ($self) = @_;
 
-    my $op = MongoDB::Op::_ListCollections->new(
+    my $op = MongoDB::Op::_ListCollections->_new(
         db_name    => $self->name,
         client     => $self->_client,
         bson_codec => $self->bson_codec,
         filter     => {},
+        options    => {},
     );
 
     my $res = $self->_client->send_read_op($op);
@@ -320,15 +320,16 @@ sub drop {
 sub run_command {
     my ( $self, $command, $read_pref ) = @_;
 
-    if ( $read_pref && ref($read_pref) eq 'HASH' ) {
-        $read_pref = MongoDB::ReadPreference->new($read_pref);
-    }
+    $read_pref = MongoDB::ReadPreference->new(
+        ref($read_pref) ? $read_pref : ( mode => $read_pref ) )
+      if $read_pref && ref($read_pref) ne 'MongoDB::ReadPreference';
 
-    my $op = MongoDB::Op::_Command->new(
-        db_name         => $self->name,
-        query           => $command,
-        bson_codec      => $self->bson_codec,
-        ( $read_pref ? ( read_preference => $read_pref ) : () ),
+    my $op = MongoDB::Op::_Command->_new(
+        db_name     => $self->name,
+        query       => $command,
+        query_flags => {},
+        bson_codec  => $self->bson_codec,
+        read_preference => $read_pref,
     );
 
     my $obj = $self->_client->send_read_op($op);
@@ -399,7 +400,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
