@@ -22,7 +22,7 @@ package MongoDB::Error;
 # ABSTRACT: MongoDB Driver Error classes
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use Moose;
 use Carp;
@@ -92,6 +92,8 @@ around BUILDARGS => sub {
     return $class->$orig(@_);
 };
 
+__PACKAGE__->meta->make_immutable;
+
 #--------------------------------------------------------------------------#
 # Subclasses with attributes included inline below
 #--------------------------------------------------------------------------#
@@ -119,6 +121,8 @@ has code => (
 
 sub _build_code { return MongoDB::Error::UNKNOWN_ERROR() }
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::DocumentError;
 use Moose;
 use MongoDB::_Types -types;
@@ -132,6 +136,8 @@ has document => (
     required => 1,
 );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::UsageError;
 use Moose;
 use MongoDB::_Types -types;
@@ -170,80 +176,120 @@ around BUILDARGS => sub {
     return $args;
 };
 
+__PACKAGE__->meta->make_immutable;
+
 #--------------------------------------------------------------------------#
 # Empty subclasses generated programatically; this keeps packages visible
 # to metadata inspectors, but is shorter than Moose/namespace::clean/extends
 #--------------------------------------------------------------------------#
 
 # Connection errors
+
 package MongoDB::ConnectionError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::HandshakeError;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::ConnectionError'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::NetworkError;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::ConnectionError'] );
 
+__PACKAGE__->meta->make_immutable;
+
 # Timeout errors
+
 package MongoDB::TimeoutError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::ExecutionTimeout;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::TimeoutError'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::NetworkTimeout;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::TimeoutError'] );
 
+__PACKAGE__->meta->make_immutable;
+
 # Database errors
+
 package MongoDB::DuplicateKeyError;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::DatabaseError'] );
 
 sub _build_code { return MongoDB::Error::DUPLICATE_KEY() }
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::NotMasterError;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::DatabaseError'] );
 
 sub _build_code { return MongoDB::Error::NOT_MASTER() }
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::WriteError;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::DatabaseError'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::WriteConcernError;
 Moose::Meta::Class->create( __PACKAGE__,
     superclasses => ['MongoDB::DatabaseError'] );
 
 sub _build_code { return MongoDB::Error::WRITE_CONCERN_ERROR() }
 
+__PACKAGE__->meta->make_immutable;
+
 # Other errors
+
 package MongoDB::AuthError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::CursorNotFoundError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::DecodingError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::GridFSError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::InternalError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::ProtocolError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 package MongoDB::SelectionError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+__PACKAGE__->meta->make_immutable;
+
 #--------------------------------------------------------------------------#
 # Private error classes
 #--------------------------------------------------------------------------#
@@ -260,6 +306,8 @@ has size => (
     required => 1,
 );
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 =pod
@@ -272,7 +320,7 @@ MongoDB::Error - MongoDB Driver Error classes
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
