@@ -20,7 +20,7 @@ package MongoDB::GridFS;
 # ABSTRACT: A file storage utility
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::GridFS::File;
 use DateTime 0.78; # drops dependency on bug-prone Math::Round
@@ -362,7 +362,7 @@ sub insert {
     while ((my $len = $fh->read(my $data, $MongoDB::GridFS::chunk_size)) != 0) {
         $chunks->insert_one({"files_id" => $id,
                                "n"        => $n,
-                               "data"     => bless(\$data)});
+                               "data"     => \$data});
         $n++;
         $length += $len;
     }
@@ -471,7 +471,7 @@ MongoDB::GridFS - A file storage utility
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
