@@ -19,14 +19,13 @@ package MongoDB::IndexView;
 # ABSTRACT: Index management for a collection
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use Moose;
 use MongoDB::Error;
 use MongoDB::WriteConcern;
-use MongoDB::_Types -types;
-use Types::Standard qw/-types slurpy/;
-use Type::Params qw/compile/;
+use MongoDB::_Types -types, qw/is_OrderedDoc is_IndexModelList/;
+use Types::Standard qw/-types is_Str/;
 use boolean;
 use namespace::clean -except => 'meta';
 
@@ -125,10 +124,9 @@ sub _build__db_name {
 my $list_args;
 
 sub list {
-    $list_args ||= compile(Object);
-    my ($self) = $list_args->(@_);
+    my ($self) = @_;
 
-    my $op = MongoDB::Op::_ListIndexes->new(
+    my $op = MongoDB::Op::_ListIndexes->_new(
         client     => $self->_client,
         db_name    => $self->_db_name,
         coll_name  => $self->_coll_name,
@@ -160,8 +158,11 @@ sub list {
 my $create_one_args;
 
 sub create_one {
-    $create_one_args ||= compile( Object, OrderedDoc, Optional( [HashRef] ) );
-    my ( $self, $keys, $opts ) = $create_one_args->(@_);
+    my ( $self, $keys, $opts ) = @_;
+
+    MongoDB::UsageError->throw("Argument to create_one must be an ordered document")
+      unless is_OrderedDoc($keys);
+
     my ($name) =
       $self->create_many( { keys => $keys, ( $opts ? ( options => $opts ) : () ) } );
     return $name;
@@ -223,12 +224,13 @@ sub create_one {
 my $create_many_args;
 
 sub create_many {
-    $create_many_args ||= compile( Object,
-        slurpy ArrayRef [ Dict [ keys => OrderedDoc, options => Optional [HashRef] ] ] );
-    my ( $self, $models ) = $create_many_args->(@_);
+    my ( $self, @models ) = @_;
 
-    my $indexes = [ map __flatten_index_model($_), @$models ];
-    my $op = MongoDB::Op::_CreateIndexes->new(
+    MongoDB::UsageError->throw("Argument to create_many must be a list of index models")
+      unless is_IndexModelList(\@models);
+
+    my $indexes = [ map __flatten_index_model($_), @models ];
+    my $op = MongoDB::Op::_CreateIndexes->_new(
         db_name       => $self->_db_name,
         coll_name     => $self->_coll_name,
         bson_codec    => $self->_bson_codec,
@@ -255,8 +257,10 @@ sub create_many {
 my $drop_one_args;
 
 sub drop_one {
-    $drop_one_args ||= compile( Object, Str );
-    my ( $self, $name ) = $drop_one_args->(@_);
+    my ( $self, $name ) = @_;
+
+    MongoDB::UsageError->throw("Argument to drop_one must be a string")
+      unless is_Str($name);
 
     if ( $name eq '*' ) {
         MongoDB::UsageError->throw("Can't use '*' as an argument to drop_one");
@@ -283,8 +287,7 @@ sub drop_one {
 my $drop_all_args;
 
 sub drop_all {
-    $drop_all_args ||= compile(Object);
-    my ($self) = $drop_all_args->(@_);
+    my ($self) = @_;
 
     return $self->collection->_run_command(
         [
@@ -368,7 +371,7 @@ MongoDB::IndexView - Index management for a collection
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
