@@ -19,21 +19,18 @@ package MongoDB::InsertOneResult;
 # ABSTRACT: MongoDB single insert result object
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
-with 'MongoDB::Role::_WriteResult';
-
-#pod =attr acknowledged
-#pod
-#pod Indicates whether this write result was ackowledged. If not, then all other
-#pod members of this result will be zero or undefined.
-#pod
-#pod =cut
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
+  MongoDB::Role::_WriteResult
+);
 
 #pod =attr inserted_id
 #pod
@@ -43,33 +40,11 @@ with 'MongoDB::Role::_WriteResult';
 
 has inserted_id => (
     is  => 'ro',
-    isa => Any,
+    required => 1,
 );
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
-#pod =method assert
-#pod
-#pod Throws an error if write errors or write concern errors occurred.
-#pod
-#pod =cut
-
-#pod =method assert_no_write_error
-#pod
-#pod Throws a MongoDB::WriteError if C<count_write_errors> is non-zero; otherwise
-#pod returns 1.
-#pod
-#pod =cut
-
-#pod =method assert_no_write_concern_error
-#pod
-#pod Throws a MongoDB::WriteConcernError if C<count_write_concern_errors> is non-zero; otherwise
-#pod returns 1.
-#pod
-#pod =cut
-
 __END__
 
 =pod
@@ -82,7 +57,7 @@ MongoDB::InsertOneResult - MongoDB single insert result object
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
@@ -98,30 +73,31 @@ This class encapsulates the result from the insertion of a single document.
 
 =head1 ATTRIBUTES
 
-=head2 acknowledged
-
-Indicates whether this write result was ackowledged. If not, then all other
-members of this result will be zero or undefined.
-
 =head2 inserted_id
 
 The identifier of the inserted document.
 
 =head1 METHODS
 
+=head2 acknowledged
+
+Indicates whether this write result was acknowledged.  Always
+true for this class.
+
 =head2 assert
 
 Throws an error if write errors or write concern errors occurred.
+Otherwise, returns the invocant.
 
 =head2 assert_no_write_error
 
-Throws a MongoDB::WriteError if C<count_write_errors> is non-zero; otherwise
-returns 1.
+Throws a MongoDB::WriteError if write errors occurred.
+Otherwise, returns the invocant.
 
 =head2 assert_no_write_concern_error
 
-Throws a MongoDB::WriteConcernError if C<count_write_concern_errors> is non-zero; otherwise
-returns 1.
+Throws a MongoDB::WriteConcernError if write concern errors occurred.
+Otherwise, returns the invocant.
 
 =head1 AUTHORS
 
