@@ -19,7 +19,7 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use Moose;
 use MongoDB;
@@ -33,6 +33,7 @@ use MongoDB::Op::_Command;
 use MongoDB::ReadPreference;
 use MongoDB::WriteConcern;
 use MongoDB::_Topology;
+use MongoDB::_Constants;
 use MongoDB::_Credential;
 use MongoDB::_URI;
 use Digest::MD5;
@@ -231,41 +232,6 @@ sub _build_connect_timeout_ms {
     );
 }
 
-#pod =attr connect_type
-#pod
-#pod Specifies the expected topology type of servers in the seed list.  The default
-#pod is 'none'.
-#pod
-#pod Valid values include:
-#pod
-#pod =for :list
-#pod * replicaSet – the topology is a replica set (ignore non replica set members
-#pod   during discovery)
-#pod * direct – the topology is a single server (connect as if the server is a
-#pod   standlone, even if it looks like a replica set member)
-#pod * none – discover the deployment topology by checking servers in the seed list
-#pod   and connect accordingly
-#pod
-#pod This may be set in a connection string with the C<connect> option.
-#pod
-#pod =cut
-
-has connect_type => (
-    is      => 'ro',
-    isa     => ConnectType,
-    lazy    => 1,
-    builder => '_build_connect_type',
-);
-
-sub _build_connect_type {
-    my ($self) = @_;
-    return $self->__uri_or_else(
-        u => 'connect',
-        e => 'connect_type',
-        d => 'none',
-    );
-}
-
 #pod =attr db_name
 #pod
 #pod Optional.  If an L</auth_mechanism> requires a database for authentication,
@@ -377,24 +343,22 @@ sub _build_local_threshold_ms {
     return $self->__uri_or_else(
         u => 'localthresholdms',
         e => 'local_threshold_ms',
-        d => 15000,
+        d => 15,
     );
 }
 
 #pod =attr max_time_ms
 #pod
 #pod Specifies the maximum amount of time in (non-negative) milliseconds that the
-#pod server should use for working on a query.  Defaults to 29,500, slightly shorter
-#pod than the L</socket_timeout_ms>, as getting a definitive (albeit negative)
-#pod response from the server is preferred over a getting a socket timeout.
-#pod
-#pod Make sure this value is shorter than C<socket_timeout_ms>.
+#pod server should use for working on a database command.  Defaults to 0, which disables
+#pod this feature.  Make sure this value is shorter than C<socket_timeout_ms>.
 #pod
 #pod B<Note>: this will only be used for server versions 2.6 or greater, as that
 #pod was when the C<$maxTimeMS> meta-operator was introduced.
 #pod
-#pod B<Also Note>: Unlike other timeout values, when this is set to zero,
-#pod C<$maxTimeMS> is disabled, not an immediate timeout.
+#pod You are B<strongly> encouraged to set this variable if you know your
+#pod environment has MongoDB 2.6 or later, as getting a definitive error response
+#pod from the server is vastly preferred over a getting a network socket timeout.
 #pod
 #pod This may be set in a connection string with the C<maxTimeMS> option.
 #pod
@@ -412,7 +376,7 @@ sub _build_max_time_ms {
     return $self->__uri_or_else(
         u => 'maxtimems',
         e => 'max_time_ms',
-        d => 29500,
+        d => 0,
     );
 }
 
@@ -538,9 +502,8 @@ sub _build_read_pref_tag_sets {
 #pod =attr replica_set_name
 #pod
 #pod Specifies the replica set name to connect to.  If this string is non-empty,
-#pod then if the topology is a replica set or a direct connection to a single
-#pod server, server replica set names must match this or they will be removed from
-#pod the topology.
+#pod then the topology is treated as a replica set and all server replica set
+#pod names must match this or they will be removed from the topology.
 #pod
 #pod This may be set in a connection string with the C<replicaSet> option.
 #pod
@@ -987,10 +950,16 @@ has _deferred => (
 
 #pod =method topology_type
 #pod
-#pod Returns an enumerated topology type.  The value will be either 'Unknown' (the
-#pod default before any servers are scanned), 'Single', 'ReplicaSetWithPrimary',
-#pod 'ReplicaSetNoPrimary' (if the primary is down or not yet discovered), or
-#pod 'Sharded'.
+#pod Returns an enumerated topology type.  If the L</replica_set_name> is
+#pod set, the value will be either 'ReplicaSetWithPrimary' or 'ReplicaSetNoPrimary'
+#pod (if the primary is down or not yet discovered).  Without L</replica_set_name>,
+#pod the type will be 'Single' if there is only one server in the list of hosts, and
+#pod 'Sharded' if there are more than one.
+#pod
+#pod N.B. A single mongos will have a topology type of 'Single', as that mongos will
+#pod be used for all reads and writes, just like a standalone mongod.  The 'Sharded'
+#pod type indicates a sharded cluster with multiple mongos servers, and reads/writes
+#pod will be distributed acc
 #pod
 #pod =cut
 
@@ -1008,24 +977,24 @@ sub _build__topology {
     my ($self) = @_;
 
     my $type =
-        $self->connect_type eq 'replicaSet' ? 'ReplicaSetNoPrimary'
-      : $self->connect_type eq 'direct'     ? 'Single'
-      :                                       'Unknown';
+        length( $self->replica_set_name ) ? 'ReplicaSetNoPrimary'
+      : @{ $self->_uri->hostpairs } > 1   ? 'Sharded'
+      :                                     'Single';
 
     MongoDB::_Topology->new(
         uri                         => $self->_uri,
         bson_codec                  => $self->bson_codec,
         type                        => $type,
         replica_set_name            => $self->replica_set_name,
-        server_selection_timeout_ms => $self->server_selection_timeout_ms,
-        local_threshold_ms          => $self->local_threshold_ms,
-        heartbeat_frequency_ms      => $self->heartbeat_frequency_ms,
+        server_selection_timeout_sec => $self->server_selection_timeout_ms / 1000,
+        local_threshold_sec          => $self->local_threshold_ms / 1000,
+        heartbeat_frequency_sec      => $self->heartbeat_frequency_ms / 1000,
         max_wire_version            => MAX_WIRE_VERSION,
         min_wire_version            => MIN_WIRE_VERSION,
         credential                  => $self->_credential,
         link_options                => {
-            connect_timeout => $self->connect_timeout_ms >= 0 ? $self->connect_timeout_ms / 1000 : -1,
-            socket_timeout  => $self->socket_timeout_ms  >= 0 ? $self->socket_timeout_ms  / 1000 : -1,
+            connect_timeout => $self->connect_timeout_ms >= 0 ? $self->connect_timeout_ms / 1000 : undef,
+            socket_timeout  => $self->socket_timeout_ms  >= 0 ? $self->socket_timeout_ms  / 1000 : undef,
             with_ssl   => !!$self->ssl,
             ( ref( $self->ssl ) eq 'HASH' ? ( SSL_options => $self->ssl ) : () ),
         },
@@ -1083,7 +1052,6 @@ sub _build__uri {
 my @deferred_options = qw(
   auth_mechanism
   auth_mechanism_properties
-  connect_type
   connect_timeout_ms
   db_name
   heartbeat_frequency_ms
@@ -1122,9 +1090,6 @@ sub BUILD {
     my $uri = $self->_uri;
 
     my @addresses = @{ $uri->hostpairs };
-    if ( $self->connect_type eq 'direct' && @addresses > 1 ) {
-        MongoDB::UsageError->throw("Connect type 'direct' cannot be used with multiple addresses: @addresses");
-    }
 
     # resolve and validate all deferred attributes
     $self->$_ for @deferred_options;
@@ -1190,6 +1155,8 @@ sub __string {
 #pod will check all servers in the deployment which ensures a connection to any
 #pod that are available.
 #pod
+#pod See L</reconnect> for a method that is useful when using forks or threads.
+#pod
 #pod =cut
 
 sub connect {
@@ -1212,6 +1179,23 @@ sub disconnect {
     return 1;
 }
 
+#pod =method reconnect
+#pod
+#pod     $client->reconnect;
+#pod
+#pod This method closes all connections to the server, as if L</disconnect> were
+#pod called, and then immediately reconnects.  Use this after forking or spawning
+#pod off a new thread.
+#pod
+#pod =cut
+
+sub reconnect {
+    my ($self) = @_;
+    $self->_topology->close_all_links;
+    $self->_topology->scan_all_servers;
+    return 1;
+}
+
 #pod =method topology_status
 #pod
 #pod     $client->topology_status;
@@ -1266,57 +1250,85 @@ sub connected {
 }
 
 sub send_admin_command {
-    my ( $self, $command, $read_preference ) = @_;
+    my ( $self, $command, $read_pref ) = @_;
 
-    my $op = MongoDB::Op::_Command->new(
-        db_name    => 'admin',
-        query      => $command,
-        bson_codec => $self->bson_codec,
-        ( $read_preference ? ( read_preference => $read_preference ) : () ),
+    $read_pref = MongoDB::ReadPreference->new(
+        ref($read_pref) ? $read_pref : ( mode => $read_pref ) )
+      if $read_pref && ref($read_pref) ne 'MongoDB::ReadPreference';
+
+    my $op = MongoDB::Op::_Command->_new(
+        db_name     => 'admin',
+        query       => $command,
+        query_flags => {},
+        bson_codec  => $self->bson_codec,
+        read_preference => $read_pref,
     );
 
     return $self->send_read_op( $op );
 }
 
+# op dispatcher written in highly optimized style
 sub send_direct_op {
     my ( $self, $op, $address ) = @_;
-    my $link = $self->_topology->get_specific_link($address);
-    return $self->_try_op( $op, $link );
+    my ( $link, $result );
+    ( $link = $self->_topology->get_specific_link($address) ), (
+        eval { ($result) = $op->execute($link); 1 } or do {
+            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+            if ( $err->$_isa("MongoDB::ConnectionError") ) {
+                $self->_topology->mark_server_unknown( $link->server, $err );
+            }
+            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
+                $self->_topology->mark_server_unknown( $link->server, $err );
+                $self->_topology->mark_stale;
+            }
+            # regardless of cleanup, rethrow the error
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+          }
+      ),
+      return $result;
 }
 
+# op dispatcher written in highly optimized style
 sub send_write_op {
     my ( $self, $op ) = @_;
-    my $link = $self->_topology->get_writable_link;
-    return $self->_try_op( $op, $link );
+    my ( $link, $result );
+    ( $link = $self->_topology->get_writable_link ), (
+        eval { ($result) = $op->execute($link); 1 } or do {
+            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+            if ( $err->$_isa("MongoDB::ConnectionError") ) {
+                $self->_topology->mark_server_unknown( $link->server, $err );
+            }
+            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
+                $self->_topology->mark_server_unknown( $link->server, $err );
+                $self->_topology->mark_stale;
+            }
+            # regardless of cleanup, rethrow the error
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+          }
+      ),
+      return $result;
 }
 
+# op dispatcher written in highly optimized style
 sub send_read_op {
     my ( $self, $op ) = @_;
-    my $link = $self->_topology->get_readable_link($op->read_preference);
-    my $type = $self->_topology->type;
-    return $self->_try_op( $op, $link, $type );
-}
-
-sub _try_op {
-    # $type might be undef; not needed for writes
-    my ($self, $op, $link, $type) = @_;
-
-    my $result = try {
-        $op->execute( $link, $type );
-    }
-    catch {
-        if ( $_->$_isa("MongoDB::ConnectionError") ) {
-            $self->_topology->mark_server_unknown( $link->server, $_ );
-        }
-        elsif ( $_->$_isa("MongoDB::NotMasterError") ) {
-            $self->_topology->mark_server_unknown( $link->server, $_ );
-            $self->_topology->mark_stale;
-        }
-        # regardless of cleanup, rethrow the error
-        die $_;
-    };
-
-    return $result;
+    my ( $link, $type, $result );
+    ( $link = $self->_topology->get_readable_link( $op->read_preference ) ),
+      ( $type = $self->_topology->type ), (
+        eval { ($result) = $op->execute( $link, $type ); 1 } or do {
+            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+            if ( $err->$_isa("MongoDB::ConnectionError") ) {
+                $self->_topology->mark_server_unknown( $link->server, $err );
+            }
+            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
+                $self->_topology->mark_server_unknown( $link->server, $err );
+                $self->_topology->mark_stale;
+            }
+            # regardless of cleanup, rethrow the error
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+          }
+      ),
+      return $result;
 }
 
 #--------------------------------------------------------------------------#
@@ -1475,7 +1487,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
@@ -1489,6 +1501,12 @@ version v0.999.999.3
         host => "mongodb://mongo.example.com:27017"
     );
 
+    # connect to a replica set (set name *required*)
+    my $client = MongoDB::MongoClient->new(
+        host => "mongodb://mongo1.example.com,mongo2.example.com",
+        replica_set_name => 'myset',
+    );
+
     my $db = $client->get_database("test");
     my $coll = $db->get_collection("people");
 
@@ -1613,31 +1631,6 @@ except when talking to a local process (and perhaps even then).
 
 This may be set in a connection string with the C<connectTimeoutMS> option.
 
-=head2 connect_type
-
-Specifies the expected topology type of servers in the seed list.  The default
-is 'none'.
-
-Valid values include:
-
-=over 4
-
-=item *
-
-replicaSet – the topology is a replica set (ignore non replica set members during discovery)
-
-=item *
-
-direct – the topology is a single server (connect as if the server is a standlone, even if it looks like a replica set member)
-
-=item *
-
-none – discover the deployment topology by checking servers in the seed list and connect accordingly
-
-=back
-
-This may be set in a connection string with the C<connect> option.
-
 =head2 db_name
 
 Optional.  If an L</auth_mechanism> requires a database for authentication,
@@ -1688,17 +1681,15 @@ This may be set in a connection string with the C<localThresholdMS> option.
 =head2 max_time_ms
 
 Specifies the maximum amount of time in (non-negative) milliseconds that the
-server should use for working on a query.  Defaults to 29,500, slightly shorter
-than the L</socket_timeout_ms>, as getting a definitive (albeit negative)
-response from the server is preferred over a getting a socket timeout.
-
-Make sure this value is shorter than C<socket_timeout_ms>.
+server should use for working on a database command.  Defaults to 0, which disables
+this feature.  Make sure this value is shorter than C<socket_timeout_ms>.
 
 B<Note>: this will only be used for server versions 2.6 or greater, as that
 was when the C<$maxTimeMS> meta-operator was introduced.
 
-B<Also Note>: Unlike other timeout values, when this is set to zero,
-C<$maxTimeMS> is disabled, not an immediate timeout.
+You are B<strongly> encouraged to set this variable if you know your
+environment has MongoDB 2.6 or later, as getting a definitive error response
+from the server is vastly preferred over a getting a network socket timeout.
 
 This may be set in a connection string with the C<maxTimeMS> option.
 
@@ -1778,9 +1769,8 @@ which case each document will be added to the tag set list.
 =head2 replica_set_name
 
 Specifies the replica set name to connect to.  If this string is non-empty,
-then if the topology is a replica set or a direct connection to a single
-server, server replica set names must match this or they will be removed from
-the topology.
+then the topology is treated as a replica set and all server replica set
+names must match this or they will be removed from the topology.
 
 This may be set in a connection string with the C<replicaSet> option.
 
@@ -1984,10 +1974,16 @@ and L</j>.
 
 =head2 topology_type
 
-Returns an enumerated topology type.  The value will be either 'Unknown' (the
-default before any servers are scanned), 'Single', 'ReplicaSetWithPrimary',
-'ReplicaSetNoPrimary' (if the primary is down or not yet discovered), or
-'Sharded'.
+Returns an enumerated topology type.  If the L</replica_set_name> is
+set, the value will be either 'ReplicaSetWithPrimary' or 'ReplicaSetNoPrimary'
+(if the primary is down or not yet discovered).  Without L</replica_set_name>,
+the type will be 'Single' if there is only one server in the list of hosts, and
+'Sharded' if there are more than one.
+
+N.B. A single mongos will have a topology type of 'Single', as that mongos will
+be used for all reads and writes, just like a standalone mongod.  The 'Sharded'
+type indicates a sharded cluster with multiple mongos servers, and reads/writes
+will be distributed acc
 
 =head2 connect
 
@@ -1998,12 +1994,22 @@ automatically as needed.  It is kept for backwards compatibility.  Calling it
 will check all servers in the deployment which ensures a connection to any
 that are available.
 
+See L</reconnect> for a method that is useful when using forks or threads.
+
 =head2 disconnect
 
     $client->disconnect;
 
 Drops all connections to servers.
 
+=head2 reconnect
+
+    $client->reconnect;
+
+This method closes all connections to the server, as if L</disconnect> were
+called, and then immediately reconnects.  Use this after forking or spawning
+off a new thread.
+
 =head2 topology_status
 
     $client->topology_status;
@@ -2153,11 +2159,16 @@ If multiple hosts are given in the seed list or discovered by talking to
 servers in the seed list, they must all be replica set members or must all be
 mongos servers for a sharded cluster.
 
-If a single, non-replica-set server is found, or if the L</connect_type> (or
-C<connect> URI option) is 'direct', the deployment is treated as a single
-server deployment.  For a replica set member, forcing the connection type to be
-'direct' routes all operations to it alone; this is useful for carrying out
-administrative activities on that server.
+A replica set B<MUST> have the C<replicaSet> option set to the replica set
+name.
+
+If there is only single host in the seed list and C<replicaSet> is not
+provided, the deployment is treated as a single server deployment and all
+reads and writes will be sent to that host.
+
+Providing a replica set member as a single host without the set name is the
+way to get a "direct connection" for carrying out administrative activities
+on that server.
 
 The connection string may also have a username and password:
 
@@ -2181,7 +2192,6 @@ The currently supported connection string options are:
 
 *authMechanism
 *authMechanism.SERVICE_NAME
-*connect
 *connectTimeoutMS
 *journal
 *readPreference
@@ -2226,7 +2236,7 @@ L</server_selection_timeout_ms> milliseconds waiting for a suitable server to
 become available.  If no server is available at the end of that time, an
 exception is thrown.
 
-=head1 SERVER MONITORING
+=head1 SERVER MONITORING AND FAILOVER
 
 When the client first needs to find a server for a database operation, all
 servers from the L</host> attribute are scanned to determine which servers to
@@ -2235,20 +2245,25 @@ discovered in this process.  Invalid hosts are dropped.
 
 After the initial scan, whenever the servers have not been checked in
 L</heartbeat_frequency_ms> milliseconds, the scan will be repeated.  This
-amortizes monitoring time over many of operations.
+amortizes monitoring time over many of operations.  Additionally, if a
+socket has been idle for a while, it will be checked before being used for
+an operation.
 
-Additionally, if a socket has been idle for a while, it will be checked
-before being used for an operation.
+If a server operation fails because of a "not master" or "node is
+recovering" error, or if there is a network error or timeout, then the
+server is flagged as unavailable and exception will be thrown.  See
+L<MongoDB::Errors> for exception types.
 
-If an server operation fails because of a "not master" or "node is recovering"
-error, the server is flagged as unavailable.  Assuming the error is caught and
-handled, the next operation will rescan all servers immediately to find a new
-primary.
-
-Whenever a server is found to be unavailable, the driver records this fact, but
-can continue to function as long as other servers are suitable per L</SERVER
+If the error is caught and handled, the next operation will rescan all
+servers immediately to update its view of the topology.  The driver can
+continue to function as long as servers are suitable per L</SERVER
 SELECTION>.
 
+When catching an exception, users must determine whether or not their
+application should retry an operation based on the specific operation
+attempted and other use-case-specific considerations.  For automating
+retries despite exceptions, consider using the L<Try::Tiny::Retry> module.
+
 =head1 AUTHENTICATION
 
 The MongoDB server provides several authentication mechanisms, though some
@@ -2386,7 +2401,7 @@ preferred, but not yet available as an OS package.  L<Authen::SASL::Cyrus>
 is available on Debian as C<libauthen-sasl-cyrus-perl> and on RHEL as
 C<perl-Authen-SASL-Cyrus>.
 
-Installing L<Authen::SASL::XS> or L<Authen::SASL::Cyrus> from CPAM requires
+Installing L<Authen::SASL::XS> or L<Authen::SASL::Cyrus> from CPAN requires
 C<libsasl>.  On Debian systems, it is available from C<libsasl2-dev>; on
 RHEL, it is available in C<cyrus-sasl-devel>.
 
@@ -2421,8 +2436,8 @@ C<auth_mechanism_properties> attribute or in the connection string.
 
 =head1 THREAD-SAFETY AND FORK-SAFETY
 
-Existing connections to servers are closed after forking or spawning a thread.  They
-will reconnect on demand.
+You B<MUST> call the L</reconnect> method on any MongoDB::MongoClient objects
+after forking or spawning a thread.
 
 =head1 AUTHORS
 
