@@ -19,13 +19,14 @@ package MongoDB::OID;
 # ABSTRACT: A Mongo Object ID
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::BSON;
-use Moose;
+use Moo;
 use MongoDB;
+use MongoDB::_Constants;
 use Types::Standard -types;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 #pod =head1 ATTRIBUTES
 #pod
@@ -40,9 +41,9 @@ use namespace::clean -except => 'meta';
 
 has value => (
     is      => 'ro',
-    isa     => Str,
     required => 1,
     builder => '_build_value',
+    isa => Str,
 );
 
 # XXX need to set up typedef with str length
@@ -56,15 +57,22 @@ sub _build_value {
 around BUILDARGS => sub {
     my $orig = shift;
     my $class = shift;
-    if (@_ == 1) {
-        return $class->$orig(value => $_[0])
-            unless ref($_[0]);
-        return $class->$orig(value => $_[0]->value)
-            if blessed($_[0]) && $_[0]->isa($class);
+    if ( @_ == 0 ) {
+        return { value => MongoDB::BSON::generate_oid() };
     }
-    return $class->$orig(@_);
+    if ( @_ == 1 ) {
+        return { value => "$_[0]" };
+    }
+    return $orig->($class, @_);
 };
 
+# This private constructor bypasses everything Moo does for us and just
+# jams an OID into a blessed hashref.  This is only for use in super-hot
+# code paths, like document insertion.
+sub _new_oid {
+    return bless { value => MongoDB::BSON::generate_oid() }, $_[0];
+}
+
 #pod =head1 METHODS
 #pod
 #pod =head2 to_string
@@ -75,10 +83,7 @@ around BUILDARGS => sub {
 #pod
 #pod =cut
 
-sub to_string {
-    my ($self) = @_;
-    $self->value;
-}
+sub to_string { $_[0]->{value} }
 
 #pod =head2 get_time
 #pod
@@ -126,9 +131,6 @@ use overload
     '""' => \&to_string,
     'fallback' => 1;
 
-
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
@@ -143,7 +145,7 @@ MongoDB::OID - A Mongo Object ID
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
