@@ -19,48 +19,50 @@ package MongoDB::Op::_Aggregate;
 # Encapsulate aggregate operation; return MongoDB::QueryResult
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 
 use MongoDB::Error;
 use MongoDB::Op::_Command;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
 use boolean;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has db_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has coll_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has client => (
     is       => 'ro',
-    isa      => InstanceOf ['MongoDB::MongoClient'],
     required => 1,
+    isa      => InstanceOf ['MongoDB::MongoClient'],
 );
 
 has pipeline => (
     is       => 'ro',
-    isa      => ArrayOfHashRef,
     required => 1,
+    isa      => ArrayOfHashRef,
 );
 
 has options => (
-    is      => 'ro',
-    isa     => HashRef,
-    default => sub { {} },
+    is       => 'ro',
+    required => 1,
+    isa      => HashRef,
 );
 
 with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
   MongoDB::Role::_ReadOp
   MongoDB::Role::_CommandCursorOp
 );
@@ -69,7 +71,7 @@ sub execute {
     my ( $self, $link, $topology ) = @_;
 
     my $options = $self->options;
-    my $is_2_6 = $link->accepts_wire_version(2);
+    my $is_2_6 = $link->does_write_commands;
 
     # maxTimeMS isn't available until 2.6 and the aggregate command
     # will reject it as unrecognized
@@ -112,9 +114,10 @@ sub execute {
         %$options,
     );
 
-    my $op = MongoDB::Op::_Command->new(
+    my $op = MongoDB::Op::_Command->_new(
         db_name         => $self->db_name,
         query           => Tie::IxHash->new(@command),
+        query_flags     => {},
         read_preference => $self->read_preference,
         bson_codec      => $self->bson_codec,
     );
@@ -124,15 +127,19 @@ sub execute {
     # For explain, we give the whole response as fields have changed in
     # different server versions
     if ( $options->{explain} ) {
-        return MongoDB::QueryResult->new(
-            _client    => $self->client,
-            address    => $link->address,
-            bson_codec => $self->bson_codec,
-            cursor     => {
-                ns         => '',
-                id         => 0,
-                firstBatch => [ $res->output ],
-            },
+        return MongoDB::QueryResult->_new(
+            _client      => $self->client,
+            address      => $link->address,
+            ns           => '',
+            bson_codec   => $self->bson_codec,
+            batch_size   => 1,
+            cursor_at    => 0,
+            limit        => 0,
+            cursor_id    => MongoDB::QueryResult::_pack_cursor_id(0),
+            cursor_start => 0,
+            cursor_flags => {},
+            cursor_num   => 1,
+            _docs        => [ $res->output ],
         );
     }
 
