@@ -20,17 +20,19 @@ package MongoDB::Op::_BulkWrite;
 # MongoDB::BulkWriteResult object
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 
 use MongoDB::BSON;
 use MongoDB::Error;
 use MongoDB::BulkWriteResult;
+use MongoDB::UnacknowledgedResult;
 use MongoDB::Op::_InsertOne;
 use MongoDB::Op::_Update;
 use MongoDB::Op::_Delete;
 use MongoDB::_Protocol;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
 use Safe::Isa;
@@ -38,53 +40,72 @@ use Scalar::Util qw/blessed reftype/;
 use Tie::IxHash;
 use Try::Tiny;
 use boolean;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has db_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has coll_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has queue => (
     is       => 'ro',
-    isa      => ArrayRef,
     required => 1,
+    isa      => ArrayRef,
 );
 
 has ordered => (
-    is      => 'ro',
-    isa     => Bool,
-    default => 1,
+    is       => 'ro',
+    required => 1,
+    isa      => Bool,
 );
 
 has write_concern => (
     is       => 'ro',
-    isa      => WriteConcern,
-    coerce   => 1,
     required => 1,
+    isa      => WriteConcern,
 );
 
 # not _WriteOp because we construct our own result objects
-with $_ for qw/MongoDB::Role::_CommandOp MongoDB::Role::_UpdatePreEncoder/;
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
+  MongoDB::Role::_CommandOp
+  MongoDB::Role::_UpdatePreEncoder
+  MongoDB::Role::_InsertPreEncoder
+);
 
 sub execute {
     my ( $self, $link ) = @_;
 
-    my $use_write_cmd = $link->accepts_wire_version(2);
+    Carp::confess("NO LINK") unless $link;
+
+    my $use_write_cmd = $link->does_write_commands;
 
     # If using legacy write ops, then there will never be a valid modified_count
     # result so we set that to undef in the constructor; otherwise, we set it
     # to 0 so that results accumulate normally. If a mongos on a mixed topology
     # later fails to set it, results merging will handle it in that case.
-    my $result =
-      MongoDB::BulkWriteResult->new( modified_count => $use_write_cmd ? 0 : undef );
+    # If unacknowledged, we have to accumulate a result to get bulk semantics
+    # right and just throw it away later.
+    my $result = MongoDB::BulkWriteResult->_new(
+        modified_count       => ( $use_write_cmd ? 0 : undef ),
+        write_errors         => [],
+        write_concern_errors => [],
+        op_count             => 0,
+        batch_count          => 0,
+        inserted_count       => 0,
+        upserted_count       => 0,
+        matched_count        => 0,
+        deleted_count        => 0,
+        upserted             => [],
+        inserted             => [],
+    );
 
     my @batches =
         $self->ordered
@@ -100,6 +121,11 @@ sub execute {
         }
     }
 
+    return MongoDB::UnacknowledgedResult->_new(
+        write_errors         => [],
+        write_concern_errors => [],
+    ) if !$self->write_concern->is_acknowledged;
+
     # only reach here with an error for unordered bulk ops
     $result->assert_no_write_error;
 
@@ -133,8 +159,28 @@ sub _execute_write_command_batch {
     while (@left_to_send) {
         my $chunk = shift @left_to_send;
 
+        # for update/insert, pre-encode docs as they need custom BSON handling
+        # that can't be applied to an entire write command at once
         if ( $cmd eq 'update' ) {
-            $chunk = $self->_pre_encode_update_chunk($link, $chunk);
+            # take array of hash, validate and encode each update doc; since this
+            # might be called more than once if chunks are getting split, check if
+            # the update doc is already encoded; this also removes the 'is_replace'
+            # field that needs to not be in the command sent to the server
+            for ( my $i = 0; $i <= $#$chunk; $i++ ) {
+                next if ref( $chunk->[$i]{u} ) eq 'MongoDB::BSON::_EncodedDoc';
+                my $is_replace = delete $chunk->[$i]{is_replace};
+                $chunk->[$i]{u} = $self->_pre_encode_update( $link, $chunk->[$i]{u}, $is_replace );
+            }
+        }
+        elsif ( $cmd eq 'insert' ) {
+            # take array of docs, encode each one while saving original or generated _id
+            # field; since this might be called more than once if chunks are getting
+            # split, check if the doc is already encoded
+            for ( my $i = 0; $i <= $#$chunk; $i++ ) {
+                unless ( ref( $chunk->[$i] ) eq 'MongoDB::BSON::_EncodedDoc' ) {
+                    $chunk->[$i] = $self->_pre_encode_insert( $link, $chunk->[$i], '.' );
+                };
+            }
         }
 
         my $cmd_doc = [
@@ -144,10 +190,11 @@ sub _execute_write_command_batch {
             writeConcern => $wc->as_struct,
         ];
 
-        my $op = MongoDB::Op::_Command->new(
-            db_name    => $db_name,
-            query      => $cmd_doc,
-            bson_codec => $self->bson_codec,
+        my $op = MongoDB::Op::_Command->_new(
+            db_name     => $db_name,
+            query       => $cmd_doc,
+            query_flags => {},
+            bson_codec  => $self->bson_codec,
         );
 
         my $cmd_result = try {
@@ -194,27 +241,11 @@ sub _execute_write_command_batch {
     return;
 }
 
-# take array of hash, validate and encode each update doc; since this
-# might be called more than once if chunks are getting split, check if
-# the update doc is already encoded; this also removes the 'is_replace'
-# field that needs to not be in the command sent to the server
-sub _pre_encode_update_chunk {
-    my ( $self, $link, $chunk ) = @_;
-    for ( my $i = 0; $i <= $#$chunk; $i++ ) {
-        next if ref( $chunk->[$i]{u} ) eq 'MongoDB::BSON::Raw';
-        my $is_replace = delete $chunk->[$i]{is_replace};
-        $chunk->[$i]{u} = $self->_pre_encode( $link, $chunk->[$i]{u}, $is_replace );
-    }
-    return $chunk;
-}
-
 sub _split_chunk {
     my ( $self, $link, $chunk, $size ) = @_;
 
-    my $max_wire_size = $self->MAX_BSON_WIRE_SIZE; # XXX blech
-
     my $avg_cmd_size       = $size / @$chunk;
-    my $new_cmds_per_chunk = int( $max_wire_size / $avg_cmd_size );
+    my $new_cmds_per_chunk = int( MAX_BSON_WIRE_SIZE / $avg_cmd_size );
 
     my @split_chunks;
     while (@$chunk) {
@@ -295,34 +326,22 @@ sub _execute_legacy_batch {
 
     for my $doc (@$docs) {
 
-        # legacy server doesn't check keys on insert; we fake an error if it
-        # happens
-
-        if ( $type eq 'insert' && ( my $r = $self->_check_no_dollar_keys($doc) ) ) {
-            if ($w_0) {
-                last if $ordered;
-            }
-            else {
-                $result->_merge_result($r);
-                $result->assert_no_write_error if $ordered;
-            }
-            next;
-        }
-
         my $op;
         if ( $type eq 'insert' ) {
-            $op = MongoDB::Op::_InsertOne->new(
+            $op = MongoDB::Op::_InsertOne->_new(
                 db_name       => $self->db_name,
                 coll_name     => $self->coll_name,
+                full_name     => $self->db_name . "." . $self->coll_name,
                 document      => $doc,
                 write_concern => $wc,
                 bson_codec    => $self->bson_codec,
             );
         }
         elsif ( $type eq 'update' ) {
-            $op = MongoDB::Op::_Update->new(
+            $op = MongoDB::Op::_Update->_new(
                 db_name       => $self->db_name,
                 coll_name     => $self->coll_name,
+                full_name     => $self->db_name . "." . $self->coll_name,
                 filter        => $doc->{q},
                 update        => $doc->{u},
                 multi         => $doc->{multi},
@@ -333,9 +352,10 @@ sub _execute_legacy_batch {
             );
         }
         elsif ( $type eq 'delete' ) {
-            $op = MongoDB::Op::_Delete->new(
+            $op = MongoDB::Op::_Delete->_new(
                 db_name       => $self->db_name,
                 coll_name     => $self->coll_name,
+                full_name     => $self->db_name . "." . $self->coll_name,
                 filter        => $doc->{q},
                 just_one      => !!$doc->{limit},
                 write_concern => $wc,
@@ -373,25 +393,4 @@ sub _execute_legacy_batch {
     return;
 }
 
-sub _check_no_dollar_keys {
-    my ( $self, $doc ) = @_;
-
-    my @keys = ref $doc eq 'Tie::IxHash' ? $doc->Keys : keys %$doc;
-    if ( my @bad = grep { substr( $_, 0, 1 ) eq '$' } @keys ) {
-        my $errdoc = {
-            index  => 0,
-            errmsg => "Document can't have '\$' prefixed field names: @bad",
-            code   => UNKNOWN_ERROR
-        };
-
-        return MongoDB::BulkWriteResult->new(
-            op_count       => 1,
-            modified_count => undef,
-            write_errors   => [$errdoc]
-        );
-    }
-
-    return;
-}
-
 1;
