@@ -19,87 +19,96 @@ package MongoDB::Op::_Delete;
 # Encapsulate a delete operation; returns a MongoDB::DeleteResult
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 
 use MongoDB::BSON;
 use MongoDB::DeleteResult;
+use MongoDB::_Constants;
 use MongoDB::_Protocol;
 use MongoDB::_Types -types;
 use Types::Standard -types;
 use Tie::IxHash;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has db_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has coll_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
+);
+
+has full_name => (
+    is       => 'ro',
+    required => 1,
+    isa      => Str,
 );
 
 has filter => (
     is       => 'ro',
-    isa      => IxHash,
-    coerce   => 1,
     required => 1,
+    isa      => Document,
 );
 
 has just_one => (
-    is      => 'ro',
-    isa     => Bool,
-    default => 1,
+    is       => 'ro',
+    required => 1,
+    isa      => Bool,
 );
 
-with qw/MongoDB::Role::_WriteOp/;
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
+  MongoDB::Role::_WriteOp
+);
 
 sub execute {
     my ( $self, $link ) = @_;
 
-    my $res =
-        $link->accepts_wire_version(2)
-      ? $self->_command_delete($link)
-      : $self->_legacy_op_delete($link);
+    my $filter =
+      ref( $self->filter ) eq 'ARRAY'
+      ? { @{ $self->filter } }
+      : $self->filter;
 
-    $res->assert;
-    return $res;
-}
+    my $op_doc = { q => $filter, limit => $self->just_one ? 1 : 0 };
 
-sub _command_delete {
-    my ( $self, $link, ) = @_;
-
-    my $op_doc = { q => $self->filter, limit => $self->just_one ? 1 : 0 };
-
-    my $cmd = Tie::IxHash->new(
-        delete       => $self->coll_name,
-        deletes      => [ $op_doc ],
-        writeConcern => $self->write_concern->as_struct,
+    return (
+        $link->does_write_commands
+        ? (
+            $self->_send_write_command(
+                $link,
+                [
+                    delete       => $self->coll_name,
+                    deletes      => [$op_doc],
+                    writeConcern => $self->write_concern->as_struct,
+                ],
+                $op_doc,
+                "MongoDB::DeleteResult"
+            )->assert
+          )
+        : (
+            $self->_send_legacy_op_with_gle(
+                $link,
+                MongoDB::_Protocol::write_delete(
+                    $self->full_name,
+                    $self->bson_codec->encode_one( $self->filter ),
+                    { just_one => $self->just_one ? 1 : 0 }
+                ),
+                $op_doc,
+                "MongoDB::DeleteResult",
+            )->assert
+        )
     );
-
-    return $self->_send_write_command( $link, $cmd, $op_doc, "MongoDB::DeleteResult" );
-}
-
-sub _legacy_op_delete {
-    my ( $self, $link ) = @_;
-
-    my $flags = { just_one => $self->just_one ? 1 : 0 };
-
-    my $ns         = $self->db_name . "." . $self->coll_name;
-    my $query_bson = $self->bson_codec->encode_one( $self->filter );
-    my $op_bson    = MongoDB::_Protocol::write_delete( $ns, $query_bson, $flags );
-    my $op_doc     = { q => $self->filter, limit => $flags->{just_one} };
-
-    return $self->_send_legacy_op_with_gle( $link, $op_bson, $op_doc, "MongoDB::DeleteResult" );
 }
 
 sub _parse_cmd {
     my ( $self, $res ) = @_;
-    return ( deleted_count => $res->{n} );
+    return ( deleted_count => $res->{n} || 0 );
 }
 
 BEGIN {
