@@ -19,19 +19,24 @@ package MongoDB::Op::_KillCursors;
 # Encapsulate a cursor kill operation; returns true
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
 use MongoDB::_Protocol;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has cursor_ids => (
     is       => 'ro',
-    isa      => ArrayRef[Str],
     required => 1,
+    isa      => ArrayRef [Str],
+);
+
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
 );
 
 sub execute {
