@@ -20,44 +20,45 @@ package MongoDB::Op::_ListCollections;
 # names
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 
 use MongoDB::Op::_Command;
 use MongoDB::Op::_Query;
 use MongoDB::QueryResult::Filtered;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
 use Tie::IxHash;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has db_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa => Str,
 );
 
 has client => (
     is       => 'ro',
-    isa      => InstanceOf['MongoDB::MongoClient'],
     required => 1,
+    isa => InstanceOf['MongoDB::MongoClient'],
 );
 
 has filter => (
     is      => 'ro',
-    isa     => IxHash,
-    coerce  => 1,
     required => 1,
+    isa => Document,
 );
 
 has options => (
     is      => 'ro',
-    isa     => HashRef,
-    default => sub { {} },
+    required => 1,
+    isa => HashRef,
 );
 
 with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
   MongoDB::Role::_ReadOp
   MongoDB::Role::_CommandCursorOp
 );
@@ -88,15 +89,21 @@ sub _command_list_colls {
         $options->{cursor} = {};
     }
 
+    my $filter =
+      ref( $self->filter ) eq 'ARRAY'
+      ? { @{ $self->filter } }
+      : $self->filter;
+
     my $cmd = Tie::IxHash->new(
         listCollections => 1,
-        filter => $self->filter,
+        filter => $filter,
         %{$self->options},
     );
 
-    my $op = MongoDB::Op::_Command->new(
+    my $op = MongoDB::Op::_Command->_new(
         db_name         => $self->db_name,
         query           => $cmd,
+        query_flags     => {},
         read_preference => $self->read_preference,
         bson_codec      => $self->bson_codec,
     );
@@ -109,7 +116,19 @@ sub _command_list_colls {
 sub _legacy_list_colls {
     my ( $self, $link, $topology ) = @_;
 
-    my $query = MongoDB::_Query->new(
+    my $query = MongoDB::_Query->_new(
+        modifiers           => {},
+        allowPartialResults => 0,
+        batchSize           => 0,
+        comment             => '',
+        cursorType          => 'non_tailable',
+        limit               => 0,
+        maxTimeMS           => 0,
+        noCursorTimeout     => 0,
+        oplogReplay         => 0,
+        projection          => undef,
+        skip                => 0,
+        sort                => undef,
         %{$self->options},
         db_name         => $self->db_name,
         coll_name       => 'system.namespaces',
