@@ -19,14 +19,14 @@ package MongoDB::QueryResult::Filtered;
 # ABSTRACT: An iterator for Mongo query results with client-side filtering
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose;
+use Moo;
 use Types::Standard -types;
 
 extends 'MongoDB::QueryResult';
 
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has post_filter => (
     is       => 'ro',
@@ -57,8 +57,6 @@ sub has_next {
     return 0;
 }
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 
 __END__
@@ -73,7 +71,7 @@ MongoDB::QueryResult::Filtered - An iterator for Mongo query results with client
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =for Pod::Coverage has_next
 
