@@ -19,22 +19,19 @@ package MongoDB::Role::_CommandOp;
 # MongoDB interface for database command operations
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::BSON;
 use MongoDB::Error;
+use MongoDB::_Constants;
 use MongoDB::_Protocol;
-use Moose::Role;
-use namespace::clean -except => 'meta';
+use Moo::Role;
+use namespace::clean;
 
 with 'MongoDB::Role::_DatabaseOp';
 
 requires qw/db_name/;
 
-use constant {
-    MAX_BSON_WIRE_SIZE => 16_793_600, # 16MiB + 16KiB
-};
-
 # XXX should pass in a read preference and modify doc accordingly
 
 sub _send_command {
