@@ -19,10 +19,10 @@ package MongoDB::Role::_Cursor;
 # Role providing a cursor interface
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose::Role;
-use namespace::clean -except => 'meta';
+use Moo::Role;
+use namespace::clean;
 
 requires qw/all has_next info next/;
 
