@@ -20,12 +20,12 @@ package MongoDB::Role::_LastError;
 # MongoDB interface for providing the last database error
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose::Role;
+use Moo::Role;
 use MongoDB::Error;
 use Syntax::Keyword::Junction qw/any/;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 # XXX should we add 'assert' to this API list?
 requires qw/last_errmsg last_code last_wtimeout/;
