@@ -19,24 +19,21 @@ package MongoDB::Role::_ReadOp;
 # MongoDB role for read ops that provides read preference
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
-use Moose::Role;
+use Moo::Role;
 
 use MongoDB::ReadPreference;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use namespace::clean -except => 'meta';
-
-my $PRIMARY = MongoDB::ReadPreference->new;
+use namespace::clean;
 
 with 'MongoDB::Role::_DatabaseOp';
 
 has read_preference => (
-    is      => 'ro',
-    isa     => ReadPreference,
-    coerce  => 1,
-    default => sub { $PRIMARY },
+    is  => 'ro',
+    isa => Maybe [ReadPreference],
 );
 
 1;
