@@ -19,17 +19,19 @@ package MongoDB::Role::_WriteOp;
 # MongoDB interface for database write operations
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::BSON;
 use MongoDB::CommandResult;
 use MongoDB::Error;
+use MongoDB::UnacknowledgedResult;
+use MongoDB::_Constants;
 use MongoDB::_Protocol;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use Moose::Role;
+use Moo::Role;
 use Syntax::Keyword::Junction qw/any/;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 use constant {
     NO_JOURNAL_RE => qr/^journaling not enabled/,
@@ -42,8 +44,8 @@ requires qw/db_name _parse_cmd _parse_gle/;
 
 has write_concern => (
     is       => 'ro',
-    isa      => WriteConcern,
     required => 1,
+    isa => WriteConcern,
 );
 
 sub _send_legacy_op_with_gle {
@@ -64,7 +66,7 @@ sub _send_legacy_op_with_gle {
 
         # errors in the command itself get handled as normal CommandResult
         if ( !$res->{ok} && ( $res->{errmsg} || $res->{'$err'} ) ) {
-            return MongoDB::CommandResult->new(
+            return MongoDB::CommandResult->_new(
                 output  => $res,
                 address => $link->address,
             );
@@ -82,7 +84,7 @@ sub _send_legacy_op_with_gle {
         if ($got_error) {
             MongoDB::DatabaseError->throw(
                 message => $got_error,
-                result => MongoDB::CommandResult->new(
+                result => MongoDB::CommandResult->_new(
                     output => $res,
                     address => $link->address,
                 ),
@@ -111,19 +113,21 @@ sub _send_legacy_op_with_gle {
             };
         }
 
-        return $result_class->new(
-            ( $write_error ? ( write_errors => [$write_error] ) : () ),
-            (
-                $write_concern_error
-                ? ( write_concern_errors => [$write_concern_error] )
-                : ()
-            ),
-            $self->_parse_gle($res, $op_doc),
+        return $result_class->_new(
+            acknowledged         => 1,
+            write_errors         => ( $write_error ? [$write_error] : [] ),
+            write_concern_errors => ( $write_concern_error ? [$write_concern_error] : [] ),
+            $self->_parse_gle( $res, $op_doc ),
         );
     }
     else {
         $link->write($op_bson);
-        return $result_class->new( acknowledged => 0 );
+        return $result_class->_new(
+            $self->_parse_gle( {}, $op_doc ),
+            acknowledged => 0,
+            write_errors => [],
+            write_concern_errors => [],
+        );
     }
 }
 
@@ -135,7 +139,7 @@ sub _send_write_command {
     if ( $self->write_concern->is_acknowledged ) {
         # errors in the command itself get handled as normal CommandResult
         if ( !$res->{ok} && ( $res->{errmsg} || $res->{'$err'} ) ) {
-            return MongoDB::CommandResult->new(
+            return MongoDB::CommandResult->_new(
                 output => $res,
                 address => $link->address,
             );
@@ -148,18 +152,18 @@ sub _send_write_command {
 
         # otherwise, construct the desired result object, calling back
         # on class-specific parser to generate additional attributes
-        return $result_class->new(
-            ( $res->{writeErrors} ? ( write_errors => $res->{writeErrors} ) : () ),
-            (
-                $res->{writeConcernError}
-                ? ( write_concern_errors => $res->{writeConcernError} )
-                : ()
-            ),
+        return $result_class->_new(
+            write_errors => ( $res->{writeErrors} ? $res->{writeErrors} : [] ),
+            write_concern_errors =>
+              ( $res->{writeConcernError} ? [ $res->{writeConcernError} ] : [] ),
             $self->_parse_cmd($res),
         );
     }
     else {
-        return $result_class->new( acknowledged => 0 );
+        return MongoDB::UnacknowledgedResult->_new(
+            write_errors => [],
+            write_concern_errors => [],
+        );
     }
 }
 
