@@ -19,54 +19,61 @@ package MongoDB::Role::_WriteResult;
 # MongoDB interface for common write result attributes and methods
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use MongoDB::Error;
+use MongoDB::_Constants;
 use MongoDB::_Types -types;
 use Types::Standard -types;
-use Moose::Role;
-use namespace::clean -except => 'meta';
-
-has acknowledged => (
-    is      => 'ro',
-    isa     => Bool,
-    default => 1,
-);
+use Moo::Role;
+use namespace::clean;
 
 has [qw/write_errors write_concern_errors/] => (
-    is      => 'ro',
-    isa     => ArrayOfHashRef,
-    coerce  => 1,
-    default => sub { [] },
+    is       => 'ro',
+    required => 1,
+    isa      => ArrayOfHashRef,
 );
 
 with 'MongoDB::Role::_LastError';
 
+sub acknowledged() { 1 }; # override to 0 for MongoDB::UnacknowledgedResult
+
+# inline assert_no_write_error and assert_no_write_concern rather
+# than having to make to additional method calls
 sub assert {
     my ($self) = @_;
-    $self->assert_no_write_error;
-    $self->assert_no_write_concern_error;
-    return 1;
+
+    $self->_throw_database_error("MongoDB::WriteError")
+      if scalar @{ $self->write_errors };
+
+    MongoDB::WriteConcernError->throw(
+        message => $self->last_errmsg,
+        result  => $self,
+        code    => WRITE_CONCERN_ERROR,
+    ) if scalar @{ $self->write_concern_errors };
+
+    return $self;
 }
 
 sub assert_no_write_error {
     my ($self) = @_;
 
     $self->_throw_database_error("MongoDB::WriteError")
-      if $self->count_write_errors;
-    return 1;
+      if scalar @{ $self->write_errors };
+
+    return $self;
 }
 
 sub assert_no_write_concern_error {
     my ($self) = @_;
-    if ( $self->count_write_concern_errors ) {
-        MongoDB::WriteConcernError->throw(
-            message => $self->last_errmsg,
-            result  => $self,
-            code    => WRITE_CONCERN_ERROR,
-        );
-    }
-    return 1;
+
+    MongoDB::WriteConcernError->throw(
+        message => $self->last_errmsg,
+        result  => $self,
+        code    => WRITE_CONCERN_ERROR,
+    ) if scalar @{ $self->write_concern_errors };
+
+    return $self;
 }
 
 sub count_write_errors {
