@@ -19,7 +19,7 @@ package MongoDB::WriteConcern;
 # ABSTRACT: Encapsulate and validate a write concern
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use Moose;
 use MongoDB::Error;
@@ -116,6 +116,8 @@ sub _w_is_acknowledged {
       && ( looks_like_number( $self->w ) ? $self->w > 0 : length $self->w );
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -130,7 +132,7 @@ MongoDB::WriteConcern - Encapsulate and validate a write concern
 
 =head1 VERSION
 
-version v0.999.999.3
+version v0.999.999.4
 
 =head1 SYNOPSIS
 
