@@ -19,7 +19,7 @@ package MongoDB::_Types;
 # MongoDB Moose type definitions
 
 use version;
-our $VERSION = 'v0.999.999.3'; # TRIAL
+our $VERSION = 'v0.999.999.4'; # TRIAL from MongoDB-v0.999.999.4-TRIAL.tar.gz
 
 use Type::Library
   -base,
@@ -33,10 +33,13 @@ use Type::Library
   CursorType
   DBRefColl
   DBRefDB
+  Document
   ErrorStr
   HashLike
   HostAddress
   HostAddressList
+  IndexModel
+  IndexModelList
   IxHash
   MongoDBCollection
   MongoDBDatabase
@@ -44,8 +47,10 @@ use Type::Library
   NonEmptyStr
   NonNegNum
   OrderedDoc
+  PairArrayRef
   ReadPrefMode
   ReadPreference
+  ServerDesc
   ServerType
   SingleChar
   SingleKeyHash
@@ -126,11 +131,16 @@ declare NonNegNum, as Num,
   where { defined($_) && $_ >= 0 },
   message { "value must be a non-negative number" };
 
+declare PairArrayRef, as ArrayRef,
+  where { @$_ % 2 == 0 };
+
 enum ReadPrefMode,
   [qw/primary primaryPreferred secondary secondaryPreferred nearest/];
 
 class_type ReadPreference, { class => 'MongoDB::ReadPreference' };
 
+class_type ServerDesc, { class => 'MongoDB::_Server' };
+
 enum ServerType,
   [
     qw/Standalone Mongos PossiblePrimary RSPrimary RSSecondary RSArbiter RSOther RSGhost Unknown/
@@ -145,13 +155,18 @@ enum TopologyType,
 
 class_type WriteConcern, { class => 'MongoDB::WriteConcern' };
 
-# after SingleKeyHash and IxHash
-declare OrderedDoc, as ArrayRef|IxHash|SingleKeyHash;
+# after SingleKeyHash, PairArrayRef and IxHash
+declare OrderedDoc, as PairArrayRef|IxHash|SingleKeyHash;
+declare Document, as HashRef|PairArrayRef|IxHash|HashLike;
 
 # after NonEmptyStr
 declare DBRefColl, as NonEmptyStr;
 declare DBRefDB, as NonEmptyStr|Undef;
 
+# after OrderedDoc
+declare IndexModel, as Dict [ keys => OrderedDoc, options => Optional [HashRef] ];
+declare IndexModelList, as ArrayRef [IndexModel];
+
 #--------------------------------------------------------------------------#
 # Coercions
 #--------------------------------------------------------------------------#
