@@ -93,6 +93,7 @@ static SV * call_method_with_pairs(SV *self, const char *method, ...);
 static SV * new_object_from_pairs(const char *klass, ...);
 static SV * _call_method_with_pairs (SV *self, const char *method, va_list args);
 static SV * call_sv_va (SV *func, int num, ...);
+static SV * call_pv_va (char *func, int num, ...);
 
 #define call_perl_reader(s,m) call_method_va(s,m,0)
 
@@ -325,6 +326,40 @@ call_sv_va (SV *func, int num, ...) {
   return ret;
 }
 
+static SV *
+call_pv_va (char *func, int num, ...) {
+  dSP;
+  SV *ret;
+  I32 count;
+  va_list args;
+
+  ENTER;
+  SAVETMPS;
+  PUSHMARK (SP);
+
+  va_start (args, num);
+  for( ; num > 0; num-- ) {
+    XPUSHs (va_arg( args, SV* ));
+  }
+  va_end(args);
+
+  PUTBACK;
+  count = call_pv(func, G_SCALAR);
+
+  SPAGAIN;
+  if (count != 1) {
+    croak ("function %s didn't return a value", func);
+  }
+  ret = POPs;
+  SvREFCNT_inc (ret);
+
+  PUTBACK;
+  FREETMPS;
+  LEAVE;
+
+  return ret;
+}
+
 /********************************************************************
  * BSON encoding
  ********************************************************************/
@@ -336,38 +371,48 @@ perl_mongo_sv_to_bson (bson_t * bson, SV *sv, HV *opts) {
     croak ("not a reference");
   }
 
-  switch (SvTYPE (SvRV (sv))) {
-  case SVt_PVHV:
-    hvdoc_to_bson (bson, sv, opts, EMPTY_STACK);
-    break;
-  case SVt_PVAV: {
-    if (sv_isa(sv, "Tie::IxHash")) {
+  if ( ! sv_isobject(sv) ) {
+    switch ( SvTYPE(SvRV(sv)) ) {
+      case SVt_PVHV:
+        hvdoc_to_bson (bson, sv, opts, EMPTY_STACK);
+        break;
+      case SVt_PVAV:
+        avdoc_to_bson(bson, sv, opts, EMPTY_STACK);
+        break;
+      default:
+        sv_dump(sv);
+        croak ("type unhandled");
+    }
+  }
+  else {
+    SV *obj;
+    char *class;
+
+    obj = SvRV(sv);
+    class = HvNAME(SvSTASH(obj));
+
+    if ( strEQ(class, "Tie::IxHash") ) {
       ixhashdoc_to_bson(bson, sv, opts, EMPTY_STACK);
     }
-    else {
-      avdoc_to_bson(bson, sv, opts, EMPTY_STACK);
-    }
-    break;
-  }
-  default:
-    if ( sv_isa(sv, "MongoDB::BSON::Raw") ) {
-        SV *str_sv;
-        char *str;
+    else if ( strEQ(class, "MongoDB::BSON::_EncodedDoc") ) {
         STRLEN str_len;
+        SV **svp;
+        SV *encoded;
+        const char *bson_str;
         bson_t *child;
-        str_sv = SvRV(sv);
-        /* check type ok */
-        if (!SvPOK(str_sv)) {
-          croak("MongoDB::BSON::Raw must be a blessed string reference");
-        }
-        str = SvPV(str_sv, str_len);
-        child = bson_new_from_data((uint8_t*) str, str_len);
+
+        encoded = _hv_fetchs_sv((HV *)obj, "bson");
+        bson_str = SvPV(encoded, str_len);
+        child = bson_new_from_data((uint8_t*) bson_str, str_len);
         bson_concat(bson, child);
         bson_destroy(child);
-        break;
     }
-    sv_dump(sv);
-    croak ("type unhandled");
+    else if (SvTYPE(obj) == SVt_PVHV) {
+      hvdoc_to_bson(bson, sv, opts, EMPTY_STACK);
+    }
+    else {
+      croak ("type (%s) unhandled", class);
+    }
   }
 }
 
@@ -432,6 +477,7 @@ _hv_to_bson(bson_t * bson, SV *sv, HV *opts, stackette *stack, bool subdoc) {
 static void
 avdoc_to_bson (bson_t * bson, SV *sv, HV *opts, stackette *stack) {
     I32 i;
+    HV* seen;
     const char *first_key = NULL;
     AV *av = (AV *)SvRV (sv);
 
@@ -444,6 +490,8 @@ avdoc_to_bson (bson_t * bson, SV *sv, HV *opts, stackette *stack) {
     /* XXX handle first key here
      */
 
+    seen = (HV *) sv_2mortal((SV *) newHV());
+
     for (i = 0; i <= av_len (av); i += 2) {
         SV **key, **val;
         STRLEN len;
@@ -453,6 +501,13 @@ avdoc_to_bson (bson_t * bson, SV *sv, HV *opts, stackette *stack) {
             croak ("failed to fetch array element");
         }
 
+        if ( hv_exists_ent(seen, *key, 0) ) {
+            croak ("duplicate key '%s' in array document", SvPV_nolen(*key));
+        }
+        else {
+            hv_store_ent(seen, *key, newSV(0), 0);
+        }
+
         str = SvPVutf8(*key, len);
         assert_valid_key(str, len);
 
@@ -462,6 +517,7 @@ avdoc_to_bson (bson_t * bson, SV *sv, HV *opts, stackette *stack) {
 
         sv_to_bson_elem (bson, str, *val, opts, EMPTY_STACK);
     }
+
 }
 
 static void
@@ -689,8 +745,19 @@ sv_to_bson_elem (bson_t * bson, const char * in_key, SV *sv, HV *opts, stackette
         bson_append_document_end(bson, &child);
       }
 
-      /* boolean */
-      else if (sv_isa(sv, "boolean")) {
+      /* boolean -- these are the most well-known boolean libraries
+       * on CPAN.  Type::Serialiser::Boolean now aliases to
+       * JSON::PP::Boolean so it is listed at the end for compatibility
+       * with old versions of it.
+       */
+      else if (
+          sv_isa(sv, "boolean") ||
+          sv_isa(sv, "JSON::XS::Boolean") ||
+          sv_isa(sv, "JSON::PP::Boolean") ||
+          sv_isa(sv, "JSON::Tiny::_Bool") ||
+          sv_isa(sv, "Mojo::JSON::_Bool") ||
+          sv_isa(sv, "Types::Serialiser::Boolean")
+        ) {
         bson_append_bool(bson, key, -1, SvIV(SvRV(sv)));
       }
       else if (sv_isa(sv, "MongoDB::Code")) {
@@ -726,22 +793,16 @@ sv_to_bson_elem (bson_t * bson, const char * in_key, SV *sv, HV *opts, stackette
       else if (sv_isa(sv, "MongoDB::MaxKey")) {
         bson_append_maxkey(bson, key, -1);
       }
-      else if (sv_isa(sv, "MongoDB::BSON::Raw")) {
-        SV *str_sv;
-        char *str;
+      else if (sv_isa(sv, "MongoDB::BSON::_EncodedDoc")) {
         STRLEN str_len;
+        SV **svp;
+        SV *encoded;
+        const char *bson_str;
         bson_t *child;
 
-        str_sv = SvRV(sv);
-
-        /* check type ok */
-        if (!SvPOK(str_sv)) {
-          croak("MongoDB::BSON::Raw must be a blessed string reference");
-        }
-
-        str = SvPV(str_sv, str_len);
-
-        child = bson_new_from_data((uint8_t*) str, str_len);
+        encoded = _hv_fetchs_sv((HV *)SvRV(sv), "bson");
+        bson_str = SvPV(encoded, str_len);
+        child = bson_new_from_data((uint8_t*) bson_str, str_len);
         bson_append_document(bson, key, -1, child);
         bson_destroy(child);
       }
@@ -773,36 +834,21 @@ sv_to_bson_elem (bson_t * bson, const char * in_key, SV *sv, HV *opts, stackette
 
         append_binary(bson, key, SvIV(subtype), data);
       }
-#if PERL_REVISION==5 && PERL_VERSION>=12
-      /* Perl 5.12 regexes */
       else if (sv_isa(sv, "Regexp")) {
+#if PERL_REVISION==5 && PERL_VERSION>=12
         REGEXP * re = SvRX(sv);
+#else
+        REGEXP * re = (REGEXP *) mg_find((SV*)SvRV(sv), PERL_MAGIC_qr)->mg_obj;
+#endif
 
         append_regex(bson, key, re, sv);
       }
-#endif
-      else if (SvTYPE(SvRV(sv)) == SVt_PVMG) {
-
-        MAGIC *remg;
-
-        /* regular expression */
-        if ((remg = mg_find((SV*)SvRV(sv), PERL_MAGIC_qr)) != 0) {
-          REGEXP *re = (REGEXP *) remg->mg_obj;
-
-          append_regex(bson, key, re, sv);
-        }
-        else {
-          /* binary */
-
-          append_binary(bson, key, BSON_SUBTYPE_BINARY, SvRV(sv));
-        }
-      }
       else if (sv_isa(sv, "MongoDB::BSON::Regexp") ) { 
         /* Abstract regexp object */
         SV *pattern, *flags;
         pattern = sv_2mortal(call_perl_reader( sv, "pattern" ));
         flags   = sv_2mortal(call_perl_reader( sv, "flags" ));
-        
+
         append_decomposed_regex( bson, key, SvPV_nolen( pattern ), SvPV_nolen( flags ) );
       }
       else {
@@ -1250,30 +1296,9 @@ bson_elem_to_sv (const bson_iter_t * iter, HV *opts ) {
     break;
   }
   case BSON_TYPE_BOOL: {
-    dSP;
-    bool d = bson_iter_bool(iter);
-    int count;
-
-    SAVETMPS;
-
-    PUSHMARK(SP);
-    PUTBACK;
-    if (d) {
-      count = call_pv("boolean::true", G_SCALAR);
-    }
-    else {
-      count = call_pv("boolean::false", G_SCALAR);
-    }
-    SPAGAIN;
-    if (count == 1)
-      value = newSVsv(POPs);
-
-    if (count != 1 || !SvOK(value)) {
-      value = newSViv(d);
-    }
-
-    PUTBACK;
-    FREETMPS;
+    value = bson_iter_bool(iter)
+      ? SvREFCNT_inc(get_sv("MongoDB::BSON::_boolean_true", GV_ADD))
+      : SvREFCNT_inc(get_sv("MongoDB::BSON::_boolean_false", GV_ADD));
     break;
   }
   case BSON_TYPE_UNDEFINED:
